/*
 * @Bubble_Map.java Version 1.1 01/24/2014
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

import java.net.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;
import javax.swing.event.*;

import cmn.cmnStruct;

import gemini.gui.geminiBubbleMapFrame;      // Bubble Map Display Dialog

import iqstrat.iqstratHeadersListStruct;     // Well Headers List Structure
import iqstrat.io.ReadHeadersXMLFile;        // Read Type Log Wells DB Table

import util.utilLandGridFile;                // Read TRS Land Grid File
import util.utilLandGridStruct;              // TRS Land Grid Data Structure
import util.utilLandGridListStruct;          // TRS Land Grid List Structure

/** Class Bubble_Map
 *  <p> This Class is the starting point for the Type Log Web App.
 *
 *  @version 1.1 01/24/2014
 *  @author  John R. Victorine
 */

public class Bubble_Map extends Applet
{
  // Runtime object

  private Runtime r = Runtime.getRuntime();

  // Web Application is runing Stand Alone Indicator

  private boolean isStandalone = false;

  private String  SQL            = new String("TYPE_LOG");
  private String  SQL_Standalone = new String("TYPE_LOG");

  // LAS Viewer Global Variables

  private cmnStruct stCMN = new cmnStruct();
  private String    sPath = new String(".");

  private iqstratHeadersListStruct stList = null; // Headers List Data Structure
  private String sCounties[] = { "", "", "", "", "" };
  private int    iCounty     = 0;
  private double dXMax       = 0.0;
  private double dXMin       = 0.0;
  private double dYMax       = 0.0;
  private double dYMin       = 0.0;

  private double dXMinTRS    = 0.0;
  private double dXMaxTRS    = 0.0;
  private double dYMinTRS    = 0.0;
  private double dYMaxTRS    = 0.0;

  /** getParameter()
   *  <p> This method wil get a parameter value
   *  @param key = The Name of the Parameter
   *  @param def = The Object Type of the Parameter
   *  @return Parameter
   */

  public String getParameter(String key, String def)
  {
    return (isStandalone ? System.getProperty(key, def) :
            (getParameter(key) != null ? getParameter(key) : def));
  }

  /** Constructor Bubble_Map()
   *  <p> This is the Constructor for this class.
   */

  public Bubble_Map() {  }

  /** Method init()
   *  <p> This Method will retrieve the data values from the HTML or the main()
   *      arguments and run the Type Log Viewer Module.
   */

  public void init()
  {
    try
    {
      SQL = this.getParameter("SQL", SQL_Standalone);
      if (SQL.equals("undefined"))
        SQL = new String("TYPE_LOG");
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }

    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will launch the Field LAS Catalog.
   * @throws Exception
   */

  private void jbInit() throws Exception
  {
	int                    i,j,k;
	String                 sCounty    = "";
	int                    iFound     = 0;
    String                 tokens[]   = null;
    String                 sDelimiter = new String("[,]+");
    utilLandGridListStruct stLandGrid = null; // Land Grid List Data
    geminiBubbleMapFrame   frame      = null;

    // initialize Global LAS Data Structure

    stCMN.bStandalone   = isStandalone;

    // Retrieve KGS Image Icon for printing

    stCMN.sApp = new String(SQL);
//System.out.println(stCMN.sApp);

    if (!isStandalone)
    {
      stCMN.urlDirCodeBase = getCodeBase();
      stCMN.appletContext  = getAppletContext();
      stCMN.pIcon          = getImage(getCodeBase(), cmn.cmnStruct.ICON);
      stCMN.pIcon2         = getImage(getCodeBase(), cmn.cmnStruct.ICON2);
    }
    else
    {
      URL url = null;

      stCMN.urlDirCodeBase =  getClass().getClassLoader().getResource(".");

      ClassLoader cl = this.getClass().getClassLoader();
      sPath = cl.getResource("images").toString();
      stCMN.sImagePath = new String(sPath);

      try
      {
        url = new URL(sPath+"\\kgsSm.png");
        stCMN.pIcon = Toolkit.getDefaultToolkit().getImage(url);
        url = new URL(sPath+"\\kgsSm2.png");
        stCMN.pIcon2 = Toolkit.getDefaultToolkit().getImage(url);
      }
      catch (java.net.MalformedURLException e)
      {
        e.printStackTrace();
        String sMessage = new String("Bubble_Map(): \n" +
                              e.getMessage() + "\n" +
                              "Couldn't create image: " + sPath+"\\kgsSm.gif");
        JOptionPane.showMessageDialog((Component) null,
                                      sMessage,
                                      "ERROR",
                                      JOptionPane.ERROR_MESSAGE);
      }
    }

    // Read the Production Lease of the area passed

    tokens = SQL.split( sDelimiter );

    if (tokens != null)
    {
      if (tokens.length == 4)
        stList = ReadProductionLeases("sLatMin="+tokens[0]+"&sLatMax="+tokens[1]+"&sLongMin="+tokens[2]+"&sLongMax="+tokens[3]);
	}

    if (SQL.length() == 10)
      stList = ReadProductionLeases( "sField=" + SQL );

//    getMinMaxPosition(); // Get the Minimum & Maximum Latitude & Longitude

    if (stList != null)
    {
      for (i=0; i<stList.iCount; i++)
	  {
 	    sCounty = new String(stList.stItem[i].sCounty.toLowerCase());
		iFound  = 0;

		for (k=0; k<iCounty; k++)
		{
		  if (sCounty.equals(sCounties[k]))
		  {
			iFound = 1;
		  }
		}

  	    if (iFound == 0)
		{
		  if (iCounty < 5)
		  {
		    sCounties[iCounty] = new String(sCounty);
		    iCounty++;
		  }
		}
	  }
	}

    stLandGrid = ReadTRSGridLines( iCounty, sCounties );
    getMinMaxPositionTRS(stLandGrid);

    getMinMaxPosition(); // Get the Minimum & Maximum Latitude & Longitude

    // Get the Path of the XML Files stored in JAR File.

    ClassLoader cl = this.getClass().getClassLoader();

    sPath = cl.getResource("images").toString();
    stCMN.sImagePath = new String(sPath);

    stCMN.cl = cl;

    // Display the LAS Viewer Start

    if ((stLandGrid != null) && (stList != null))
    {
      frame = new geminiBubbleMapFrame(400, "Production Bubble Map", stCMN, stLandGrid, stList, 0);
      frame.setArea( dXMin, dXMax, dYMin, dYMax );
	}
  }

  /* ======================================================================= *
   * ----------------------------- GET METHODS ----------------------------- *
   * ======================================================================= */

  /** Method getMinMaxPosition()
   *  <p> This method will determine the minimum and maximum position of the
   *      wells that have been passed in.
   */

  private void getMinMaxPosition()
  {
	int    iStart     = 0;
    double dMinLat    = 0.0;
    double dMaxLat    = 0.0;
    double dMinLong   = 0.0;
    double dMaxLong   = 0.0;
    double dLongDelta = 0.0;
    double dLatDelta  = 0.0;

	if (stList != null)
	{
	  for (int i=0; i<stList.iCount; i++)
	  {
        if (iStart == 0)
        {
          dMinLat  = stList.stItem[i].dLatitude;
          dMaxLat  = stList.stItem[i].dLatitude;
          dMinLong = stList.stItem[i].dLongitude;
          dMaxLong = stList.stItem[i].dLongitude;
          iStart   = 1;
        }

        if ((dMinLat > stList.stItem[i].dLatitude) &&
            (stList.stItem[i].dLatitude >= dYMinTRS) &&
            (stList.stItem[i].dLatitude <= dYMaxTRS))
          dMinLat  = stList.stItem[i].dLatitude;

        if ((dMaxLat < stList.stItem[i].dLatitude) &&
            (stList.stItem[i].dLatitude >= dYMinTRS) &&
            (stList.stItem[i].dLatitude <= dYMaxTRS))
          dMaxLat  = stList.stItem[i].dLatitude;

        if ((dMinLong > stList.stItem[i].dLongitude) &&
            (stList.stItem[i].dLongitude >= dXMinTRS) &&
            (stList.stItem[i].dLongitude <= dXMaxTRS))
          dMinLong  = stList.stItem[i].dLongitude;

        if ((dMaxLong < stList.stItem[i].dLongitude) &&
            (stList.stItem[i].dLongitude >= dXMinTRS) &&
            (stList.stItem[i].dLongitude <= dXMaxTRS))
          dMaxLong  = stList.stItem[i].dLongitude;
	  }
	}

    dLongDelta = 0.05 * (dMaxLong - dMinLong);
    dLatDelta  = 0.05 * (dMaxLat  - dMinLat);

    dXMin      = dMinLong - dLongDelta;
    dXMax      = dMaxLong + dLongDelta;
    dYMin      = dMinLat  - dLatDelta;
    dYMax      = dMaxLat  + dLatDelta;

//System.out.println("Longitude: "+dXMin+" "+dXMax+" <-> Latitude: "+dYMin+" "+dYMax);
  }

  /** Method getMinMaxPositionTRS()
   *  <p> This method will determine the minimum and maximum position of the
   *      wells that have been passed in.
   */

  private void getMinMaxPositionTRS(utilLandGridListStruct st)
  {
	int    iStart     = 0;
    double dMinLat    = 0.0;
    double dMaxLat    = 0.0;
    double dMinLong   = 0.0;
    double dMaxLong   = 0.0;
    double dLongDelta = 0.0;
    double dLatDelta  = 0.0;

	if (st != null)
	{
	  for (int i=0; i<st.iCount; i++)
	  {
		for (int j=0; j<st.stItem[i].iTotal; j++)
		{
          if (iStart == 0)
          {
            dMinLat  = st.stItem[i].dLatitude[j];
            dMaxLat  = st.stItem[i].dLatitude[j];
            dMinLong = st.stItem[i].dLongitude[j];
            dMaxLong = st.stItem[i].dLongitude[j];
            iStart   = 1;
          }

          if (dMinLat > st.stItem[i].dLatitude[j])
            dMinLat  = st.stItem[i].dLatitude[j];

          if (dMaxLat < st.stItem[i].dLatitude[j])
            dMaxLat  = st.stItem[i].dLatitude[j];

          if (dMinLong > st.stItem[i].dLongitude[j])
            dMinLong  = st.stItem[i].dLongitude[j];

          if (dMaxLong < st.stItem[i].dLongitude[j])
            dMaxLong  = st.stItem[i].dLongitude[j];
		}
	  }

      dLongDelta = 0.05 * (dMaxLong - dMinLong);
      dLatDelta  = 0.05 * (dMaxLat  - dMinLat);

      dXMinTRS   = dMinLong - dLongDelta;
      dXMaxTRS   = dMaxLong + dLongDelta;
      dYMinTRS   = dMinLat  - dLatDelta;
      dYMaxTRS   = dMaxLat  + dLatDelta;
	}

//System.out.println("TRS: Longitude: "+dXMinTRS+" "+dXMaxTRS+" <-> Latitude: "+dYMinTRS+" "+dYMaxTRS);
  }

  /* ======================================================================= *
   * ----------------------- READ Production Leases ------------------------ *
   * ======================================================================= */

  /** Method ReadProductionLeases()
   * <p> This method will search the KGS Database for CO2 Leases.
   *  @param sQuery  = The county string name or Area Number
   */

  private static iqstratHeadersListStruct ReadProductionLeases( String sQuery )
  {
	iqstratHeadersListStruct st = null;
    String sql = cmn.cmnStruct.GET_LEASE_PRODUCTION_XML;

    st = getData(iqstrat.io.ReadHeadersXMLFile.URL, sql + sQuery);

    return (st);
  }

  /** Method getData()
   * <p> This method will read the User information xml data
   * @param  iMethod = The Method used in retrieving the XML
   * @param  sPath   = The url or File Path to use
   * @return stUser  = The User Data Structure
   */

  public static iqstratHeadersListStruct getData(int iMethod, String sPath)
  {
    iqstratHeadersListStruct st = null; // headers Information Data Structure

    ReadHeadersXMLFile pLocInfo = new ReadHeadersXMLFile(iMethod);
    st = pLocInfo.Process(sPath);

    String sError = pLocInfo.GetError();
    if (sError.length() > 0)
      JOptionPane.showMessageDialog( (Component)null,
                                     sError,
                                     "ERROR",
                                     JOptionPane.ERROR_MESSAGE);

    return (st);
  }

  /* ======================================================================= *
   * ------------------------ READ TRS Grid Lines -------------------------- *
   * ======================================================================= */

  /** Method ReadTRSGridLines()
   *  <p> This method will read the TRS Grid Lines for plotting
   *  @param  iCounty   = The number of Counties detected
   *  @param  sCounties = the counties list
   *  @return stGrid    = the TRS Grid Line List Structure
   */

  private static utilLandGridListStruct ReadTRSGridLines( int iCounty, String sCounties[] )
  {
	int                    i,j,k;
    utilLandGridListStruct stGrid = null;
    utilLandGridListStruct st     = null;
    utilLandGridFile       pGrid  = null;

    if (iCounty > 0)
    {
      if (iCounty == 1)
      {
//System.out.println(cmn.cmnStruct.KGS + cmn.cmnStruct.LANDGRID +"\\"+ sCounties[0]+".dat");
        pGrid = new  utilLandGridFile(
                           util.utilFileIO.URL_FILE,
                           cmn.cmnStruct.KGS + cmn.cmnStruct.LANDGRID,
                           sCounties[0]+".dat");
        stGrid = util.utilLandGridUtility.transfer( pGrid.getLandGridList() );
      }
      else
      {
        for (i=0; i<iCounty; i++)
        {
          if (pGrid != null)
            pGrid.delete();
          pGrid = null;

//System.out.println(cmn.cmnStruct.KGS + cmn.cmnStruct.LANDGRID +"\\"+ sCounties[i]+".dat");

          pGrid = new  utilLandGridFile(
                             util.utilFileIO.URL_FILE,
                             cmn.cmnStruct.KGS + cmn.cmnStruct.LANDGRID,
                             sCounties[i].toLowerCase()+".dat");
          st = pGrid.getLandGridList();

          stGrid = util.utilLandGridUtility.addCounty(stGrid, st);

          if (st != null)
            st.delete();
          st = null;
        }
	  }

      if (pGrid != null)
        pGrid.delete();
      pGrid = null;
    }

    return (stGrid);
  }

  /* ======================================================================= *
   * --------------------------- APPLET METHODS ---------------------------- *
   * ======================================================================= */

  /** Method start()
   *  <p> This method will start the applet
   */

  public void start() { }

  /** Method stop()
   *  <p> This method will stop the applet
   */

  public void stop() { }

  /** Method destroy()
   *  <p> This method will destroy the applet
   */

  public void destroy() { }

  /** Method getAppletInfo()
   *  <p> This method will return Applet information
   *  @return - Type Log Plot
   */

  public String getAppletInfo() { return ("LAS IO Applet"); }

  /** Method getParameterInfo()
   *  <p> This method will get the parameter info
   *  @return pinfo - Array of Parameters Expected.
   */

  public String[][] getParameterInfo()
  {
    String[][] pinfo = { {"SQL",  "String", "Application to run"}, };

    return pinfo;
  }

  /** Method main()
   *  <p> This method will allow this Module to run either as an Application or
   *      an Applet.
   *  @param args = The Arguments passed in to query for the production data
   */

  public static void main(String[] args)
  {
    Bubble_Map applet = new Bubble_Map();

    applet.isStandalone = true;
    applet.SQL_Standalone = new String("TYPE_LOG");
    if (args.length > 0)
      applet.SQL_Standalone = new String(args[0]);

    applet.init();
    applet.start();
  }
}

/*
 *  @version 1.1 01/24/2014
 *  @author  John Victorine
 */
