/*
 * @geminiAreaControlFrame.java Version 1.1 10/01/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package gemini.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import cmn.cmnStruct;                    // GLOBAL Common Data Structure

import plot.plotWellSymbolFrame;         // Plot Well Symbols Dialog

import iqstrat.iqstratHeadersListStruct; // IQSTRAT Headers List Structure
import iqstrat.iqstratHeadersStruct;     // IQSTRAT Headers Data Structure

import gemini.gui.geminiAreaMapFrame;    // Area Map Plot Frame

import util.utilLandGridListStruct;      // TRS County Land Grid List Structure

/** Class geminiAreaControlFrame
 *  <p> This file will display the Kansas Oil & Gas Wells.
 *
 *  @version 1.1 10/01/2012
 *  @author  John R. Victorine
 */

public class geminiAreaControlFrame extends JFrame implements ActionListener, Observer
{
  // Input Variables

  private Observable                pNotifier = null; // Observable Object previous Frame
  private String                    sTitle    = "";   // Title
  private cmnStruct                 stCMN     = null; // Global Common Data Structure
  private utilLandGridListStruct    stGrid    = null; // Land Grid List Data Structure
  private iqstratHeadersListStruct  stWells   = null; // Well List Data Structure
  private int                       iGrid     = -1;   // Grid Type

  // Global Variables

  private Observable                notifier  = null; // Observable Object this Frame
  private geminiAreaMapFrame        plot      = null; // Interactive Map
  private plotWellSymbolFrame       pSymbols  = null; // Well Symbol Frame

  private int                       iFields     = 0;    // Total # of Fields
  private String                    sFields[][] = null; // List of FIelds

  private double                    dMinLatitude  = 0.0; // Minimum Latitude
  private double                    dMaxLatitude  = 0.0; // Maximum Latitude
  private double                    dMinLongitude = 0.0; // Minimum Longitude
  private double                    dMaxLongitude = 0.0; // Maximum Longitude

  private String                    sField        = "0"; // Field KID

  // Well Data

  private iqstratHeadersListStruct stList     = null;

  // Screen Width

  private int iControlWidth = 400;  // Control Panel Width
  private int iScreenWidth  = 0;    // Screen Width
  private int iScreenHigh   = 0;    // Scree Hieght

  // Global Widgets

  private JRadioButton rbOFF     = new JRadioButton();
  private JRadioButton rbZoomIn  = new JRadioButton();
  private JRadioButton rbZoomOut = new JRadioButton();

  // -- Add Text to Map

  private static final int _SYMBOL     = 0;
  private static final int _WELL_NAME  = 1;
  private static final int _API_NUMBER = 2;

  private int iADD = _SYMBOL;

  // -- Well Information Data

  public int    iWells     = 0;   // Total Wells Selected

  public static final int _TOTAL_WELLS = 4;
  private int iSelected = 0;

  private JRadioButton rbField     = new JRadioButton();
  private JRadioButton rb[]        = null;

  private JButton      btnContinue = new JButton();

  private JButton      btnArea  = new JButton();
  private JButton      btnField = new JButton();

  // -- Menu Options
  // .. File Button Menu

  private JMenuItem mExit    = null;  // Exit

  // .. Well Symbol Status

  private JMenuItem mStatus  = null;  // Well Status

  /** Constructor geminiAreaControlFrame()
   *  <p> This is the Constructor for this class
   *  @param pNotifier = Observable Object previous Frame
   *  @param sTitle    = Title
   *  @param stStruct  = Global IQSTRAT Data Structure
   *  @param stGrid    = Utility Land Grid List Data Structure
   *  @param stWells   = Well Header List Data Structure
   *  @param iGrid     = Plot Text with Grid or NOT.
   */

  public geminiAreaControlFrame( Observable               pNotifier,
	                             String                   sTitle,
	                             cmnStruct                stCMN,
                                 utilLandGridListStruct   stGrid,
                                 iqstratHeadersListStruct stWells,
                                 int iGrid  )
  {
    try
    {
	  this.pNotifier = pNotifier;
      this.sTitle    = sTitle;
      this.stGrid    = stGrid;
      this.stWells   = stWells;
      this.iGrid     = iGrid;

      this.stCMN     = stCMN;

      getFields();

      jbInit();

      // Display the Interactive Map Frame

      plot = new geminiAreaMapFrame( iControlWidth, notifier, sTitle,
                                     stCMN, stGrid, stWells, iGrid );

      setButtons();

      addWindowListener( new geminiAreaControlFrame_WindowListener() );
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
      String sMessage = new String("geminiAreaControlFrame(): " + ex.getMessage());
      JOptionPane.showMessageDialog((Component) null,
                                    sMessage,
                                    "ERROR",
                                    JOptionPane.ERROR_MESSAGE);
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Panel.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	JPanel pnlTop      = new JPanel();
    JPanel pnlLeft     = new JPanel();
    JPanel pnlRight    = new JPanel();

	JPanel pnlCenter   = new JPanel();

	JPanel pnlData     = new JPanel();
	JPanel panels[]    = null;

	JPanel pnlBase     = new JPanel();

    JScrollPane scroll = new JScrollPane();

    ButtonGroup  groupZoom   = new ButtonGroup();
    ButtonGroup  groupField  = new ButtonGroup();
    ButtonGroup  groupFilter = new ButtonGroup();
    ButtonGroup  groupText   = new ButtonGroup();
/*
    TitledBorder titledBorderTop = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Filter Wells By:");
    titledBorderTop.setTitleFont(new java.awt.Font("Dialog", 1, 14));

    TitledBorder titledBorderWells = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Step 1: Select Wells in Map");
    titledBorderWells.setTitleFont(new java.awt.Font("Dialog", 1, 14));
    titledBorderWells.setTitleColor(Color.blue);
*/
    TitledBorder titledBorderFields = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Fields:");
    titledBorderFields.setTitleFont(new java.awt.Font("Dialog", 1, 14));

    // Build Menus
    // -- Menu Bar
    // .. File Menu

    JMenuBar menuBar    = new JMenuBar();
    JMenu    menuFile   = new JMenu("File");

    mExit = new JMenuItem( "Exit" );
    mExit.addActionListener(this);

    menuFile.add( mExit );     // Exit Dialog

    menuBar.add(menuFile);

    // .. Well Symbol Status Menu

    JMenu    menuStatus = new JMenu("Well Status Symbols");

    mStatus = new JMenuItem( "Display Well Symbols" );
    mStatus.addActionListener(this);

    menuStatus.add( mStatus );     // Well Status Dialog

    menuBar.add(menuStatus);

    // -- Add Menu Bar

    this.setJMenuBar(menuBar);

    // Create an Observable

    notifier = new geminiAreaControlFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver( this );

    // Build Frame

    this.setTitle( "Control" );
    this.getContentPane().setLayout(new BorderLayout());

    // -- Top Panel

//    pnlTop.setLayout(new BorderLayout());
//    pnlTop.setBorder( titledBorderTop );

    rbOFF.setSelected(true);
    rbOFF.setFont(new java.awt.Font("Dialog", 1, 11));
	rbOFF.setText("Zoom Off");
	rbOFF.addActionListener(this);

    rbZoomIn.setFont(new java.awt.Font("Dialog", 1, 11));
	rbZoomIn.setText("Zoom In");
	rbZoomIn.addActionListener(this);

    rbZoomOut.setFont(new java.awt.Font("Dialog", 1, 11));
	rbZoomOut.setText("Zoom Out");
	rbZoomOut.addActionListener(this);

    // -- Center Panel

    pnlCenter.setLayout(new BorderLayout());

    rbField.setSelected(true);
    rbField.setFont(new java.awt.Font("Dialog", 1, 11));
	rbField.setText("None");
	rbField.addActionListener(this);

    groupField.add( rbField );

    pnlCenter.add(rbField, BorderLayout.NORTH);
    pnlCenter.setBorder( titledBorderFields );

    if (iFields > 0)
    {
      pnlData.setLayout(new GridLayout(iFields, 1));

      rb = new JRadioButton[iFields];

      for (int k=0; k<iFields; k++)
      {
		rb[k] = new JRadioButton();
        rb[k].setFont(new java.awt.Font("Dialog", 1, 11));
	    rb[k].setText( sFields[k][1] );
	    rb[k].addActionListener(this);

	    groupField.add( rb[k] );
        pnlData.add(rb[k], null);
	  }
	}

    // -- Base Panel

    pnlBase.setLayout(new GridLayout());

    btnContinue.setFont(new java.awt.Font("Dialog", 1, 13));
    btnContinue.setText("Step 2: Load Type Log Data");
    btnContinue.setForeground( Color.blue );
    btnContinue.addActionListener(this);

    btnArea.setFont(new java.awt.Font("Dialog", 1, 13));
    btnArea.setText("Display By Area");
    btnArea.addActionListener(this);

    btnField.setFont(new java.awt.Font("Dialog", 1, 13));
    btnField.setText("Display By Field");
    btnField.addActionListener(this);

    // Add Widgets to Frame

    this.getContentPane().add(pnlTop,    BorderLayout.NORTH);
    pnlTop.add(rbOFF,                    null);
    pnlTop.add(rbZoomIn,                 null);
    pnlTop.add(rbZoomOut,                null);

    groupZoom.add(rbOFF);
    groupZoom.add(rbZoomIn);
    groupZoom.add(rbZoomOut);

    this.getContentPane().add(pnlCenter, BorderLayout.CENTER);

    pnlCenter.add(scroll,                BorderLayout.CENTER);
    scroll.getViewport().add(pnlData,    null);

    this.getContentPane().add(pnlBase,   BorderLayout.SOUTH);
    pnlBase.add(btnArea,                 null);
    pnlBase.add(btnField,               null);
//    pnlBase.add(btnContinue,             BorderLayout.CENTER);

    // Display the Frame

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    iScreenWidth = d.width;
    iScreenHigh  = d.height-50;

    this.setSize(new Dimension(iControlWidth, iScreenHigh)); //700));
    this.setLocation(1,1);

//    this.setResizable(false);
    this.setVisible(true);
  }

  /** METHOD geminiAreaControlFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class geminiAreaControlFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      close();
    }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    pNotifier = null; // Observable Object previous Frame
    sTitle    = null; // Title
    stCMN     = null; // Global Common Data structure
    stGrid    = null; // Land Grid List Data structure
    stWells   = null; // Well List Data structure

    // Global Variables

    notifier  = null; // Observable Object this Frame

    if (plot != null)
      plot.close();
    plot = null;

    if (pSymbols != null)
      pSymbols.dispose();

    // Global Widgets

    rbOFF       = null;
    rbZoomIn    = null;
    rbZoomOut   = null;

    btnArea     = null;
    btnField    = null;
    btnContinue = null;

    // File Button Menu

    mExit    = null;  // Exit
    mStatus  = null;  // Well Status

	dispose();
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getFields()
   *  <p> This method will get the list of fields from the Well List
   */

  private void getFields()
  {
	int iCont = 0;

	if (stWells != null)
	{
	  if (stWells.iCount > 0)
	  {
		sFields = new String[stWells.iCount][2];

		for (int i=0; i<stWells.iCount; i++)
		{
	      if (!stWells.stItem[i].sField_kid.equals("0"))
		  {
			iCont = 0;
		    for (int k=0; k<iFields; k++)
		    {
		      if (stWells.stItem[i].sField_kid.equals(sFields[k][0]))
		      {
				iCont = 1;
			  }
			}

			if (iCont == 0)
			{
		      sFields[iFields][0] = new String(stWells.stItem[i].sField_kid);
		      sFields[iFields][1] = new String(stWells.stItem[i].sField);
		      iFields++;
			}
		  }
		}

		sFields = bubbleSort( iFields, sFields );
	  }
	}
  }

  /** Method bubbleSort()
   * <p> This method will sort in ascending depth order (lowest to highest)
   * @param  st = stratigraphic unit data list structure
   * @return st = sorted stratigraphic unit data list structure
   */

  public static String[][] bubbleSort( int icnt, String str[][] )
  {
    boolean swappedOnPrevRun  = true;
    String  sTemp[]           = {"", ""};

    if (str != null)
    {
      if (icnt > 1)
      {
        while(swappedOnPrevRun)
        {
          // this variable keeps track of whether to continue sorting or exit

          swappedOnPrevRun = false;

          // loop through every element in the array, except for the last one

          for(int i=0; i<icnt-1; i++)
          {
            // if current element is greater than the next swap the two elements

            if (str[i][1].compareToIgnoreCase(str[i+1][1]) > 0)
            {
              // we don't want the loop to end just yet, we're not done

              swappedOnPrevRun = true;

              // store element i in a temporary variable

              sTemp[0] = new String( str[i][0] );
              sTemp[1] = new String( str[i][1] );

              // set element i+1 to where i used to be

              str[i][0] = new String( str[i+1][0] );
              str[i][1] = new String( str[i+1][1] );

              // release the old i from temp into i+1 slot

              str[i+1][0] = new String( sTemp[0] );
              str[i+1][1] = new String( sTemp[1] );
            }
          }
        }
      }
    }

    return (str);
  }

  /** Method getWellCount()
   *  <p> This method will retrieve the total well count
   *  @return iWells = Total wells selected
   */

  public int getWellCount() { return (iWells); }

  /** Method getWellList()
   *  <p> This method will return the list of wells selected
   *  @return stList = Well Headers List Structure of selected wells
   */

  public iqstratHeadersListStruct getWellList() { return (stList); }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setButtons()
   *  <p> This method will enable & disable buttons
   */

  private void setButtons()
  {
    btnArea.setEnabled(false);
    btnField.setEnabled(false);

    if (!sField.equals("0"))
      btnField.setEnabled(true);

    if ((dMinLatitude  != 0.0) && (dMaxLatitude  != 0.0) &&
        (dMinLongitude != 0.0) && (dMaxLongitude != 0.0))
    {
      btnArea.setEnabled(true);
    }
  }

  /** Method setLogType()
   *  <p> This method will set the filter of the wells that are
   *      being plotted.
   *  @param filter = the filter to the Well plot
   */

  public void setLogType(String filter) { plot.setLogType(filter); }

  /** Method setText()
   *  <p> This method will set the filter of the wells that are
   *      being plotted.
   *  @param filter = the filter to the Well plot
   */

  public void setText(int iAdd )
  {
	this.iADD = iAdd;
	plot.setText(iADD);
  }

  /** Method setField
   *  <p> This method will set the Field KID
   */

  private void setField()
  {
	if (rbField.isSelected())
	{
      plot.setField("0");
      sField = new String("0"); // Field KID
	}

	for (int k=0; k<iFields; k++)
	{
	  if (rb[k].isSelected())
	  {
	    plot.setField(sFields[k][0]);
        sField = new String(sFields[k][0]); // Field KID
	  }
    }
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == rbOFF) { plot.zoom_off(); }

    if (event.getSource() == rbZoomIn)
    {
	  plot.zoom_in();

      dMinLatitude  = plot.getMinLatitude();  // Minimum Latitude
      dMaxLatitude  = plot.getMaxLatitude();  // Maximum Latitude
      dMinLongitude = plot.getMinLongitude(); // Minimum Longitude
      dMaxLongitude = plot.getMaxLongitude(); // Maximum Longitude
	}

    if (event.getSource() == rbZoomOut)
    {
      plot.zoom_out();
      rbOFF.setSelected(true);

      dMinLatitude  = 0.0; // Minimum Latitude
      dMaxLatitude  = 0.0; // Maximum Latitude
      dMinLongitude = 0.0; // Minimum Longitude
      dMaxLongitude = 0.0; // Maximum Longitude
    }

    if (event.getSource() == rbField)
    {
  	  setField();
	}

    for (int i=0; i<iFields; i++)
    {
	  if (event.getSource() == rb[i])
	  {
		setField();
	  }
	}

    if (event.getSource() == btnArea)
    {
      System.out.println("sLatMin="   + dMinLatitude  +
                         "&sLatMax="  + dMaxLatitude  +
                         "&sLongMin=" + dMinLongitude +
                         "&sLongMax=" + dMaxLongitude);
	}

    if (event.getSource() == btnField)
    {
	  System.out.println("sField=" + sField);
    }

	if (event.getSource() == btnContinue)
	{
//	  if (stList != null)
//	    stList.delete();
//	  stList = null;

//      for (int i=0; i<iWells; i++)
//        stList = iqstrat.iqstratHeadersUtility.add( stList, stData[i] );

//      if (pNotifier != null)
//         pNotifier.notifyObservers(new String("Load Well Data"));
	}

    if (event.getSource() == mStatus)
    {
      if (pSymbols != null)
        pSymbols.dispose();

      pSymbols = new plotWellSymbolFrame(320, 470);
	}

    setButtons();

    if (pNotifier != null)
      pNotifier.notifyObservers( new String( "FREE MEMORY" ) );

    if (event.getSource() == mExit)  { close(); } // Exit
  }

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    int    icnt = -1;
    int    iW   = -1;
    String sArg = new String( (String) arg);

    if (sArg.equals("Close Map Area Plot")) { close(); }

    if (pNotifier != null)
      pNotifier.notifyObservers( new String( "FREE MEMORY" ) );

    setButtons();
  }
}


/** CLASS geminiAreaControlFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class geminiAreaControlFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 10/01/2012
 *  @author  John Victorine
 */
