/*
 * @geminiAreaMapFrame.java Version 1.1 09/27/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package gemini.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import cmn.cmnStruct;
import iqstrat.iqstratHeadersListStruct;
import iqstrat.iqstratHeadersStruct;
import gemini.plot.geminiPlotWells;
import util.utilLandGridListStruct;

/** Class geminiKSMapCountyFrame
 *  <p> This file will display the Kansas Oil & Gas Wells.
 *
 *  @version 1.1 09/27/2012
 *  @author  John R. Victorine
 */

public class geminiAreaMapFrame extends JFrame implements ActionListener, Observer
{
  private Observable               pNotifier = null; // Observable Object this Frame
  private String                   sTitle    = "";
  private cmnStruct                stStruct  = null; // Global Common Data
  private utilLandGridListStruct   stGrid    = null; // Land Grid List Data
  private iqstratHeadersListStruct stWells   = null; // Well List Data
  private int                      iGrid     = -1;   // Grid

  // Global Variables

  private Observable      notifier = null; // Observable Object this Frame
  private geminiPlotWells panel    = null; // Plot Wells Panel

  private iqstratHeadersStruct stWell = null; // Selected Well

  // Screen Width

  private int iControlWidth = 0;  // Control Panel Width
  private int iScreenWidth  = 0;    // Screen Width
  private int iScreenHigh   = 0;    // Scree Hieght

  // Global Widgets

  /** Constructor geminiKSMapCountyPanel()
   *  <p> This is the Constructor for this class
   *  @param sTitle   - Title
   *  @param stStruct - Global Common Data Structure
   *  @param stGrid   - Utility Land Grid List Data Structure
   *  @param stWell   - Well Header List Data Structure
   *  @param iGrid    - Plot Text with Grid or NOT.
   */

  public geminiAreaMapFrame( int                      iControlWidth,
                             Observable               pNotifier,
                             String                   sTitle,
	                         cmnStruct                stStruct,
                             utilLandGridListStruct   stGrid,
                             iqstratHeadersListStruct stWells,
                             int iGrid  )
  {
    try
    {
	  this.iControlWidth = iControlWidth;
	  this.pNotifier     = pNotifier;
      this.sTitle        = sTitle;
      this.stStruct      = stStruct;
      this.stGrid        = stGrid;
      this.stWells       = stWells;
      this.iGrid         = iGrid;

      jbInit();

      addWindowListener( new geminiAreaControlFrame_WindowListener() );
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
      String sMessage = new String("geminiKSMapCountyPanel(): " + ex.getMessage());
      JOptionPane.showMessageDialog((Component) null,
                                    sMessage,
                                    "ERROR",
                                    JOptionPane.ERROR_MESSAGE);
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Panel.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	JPanel   pnlTop     = new JPanel();
	JLabel   lbl        = new JLabel();
	int      iWidth     = 0;
	int      iHeight    = 0;

    // Create an Observable

    notifier = new geminiAreaMapFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver( this );

    // Build Frame

    this.setTitle( sTitle );
    this.getContentPane().setLayout(new BorderLayout());

    pnlTop.setBackground(Color.white);

    lbl.setText("");
	lbl.setHorizontalTextPosition(JLabel.LEFT);
    lbl.setBackground(Color.white);

    Dimension d  = Toolkit.getDefaultToolkit().getScreenSize();
    iScreenWidth = d.width;
    iScreenHigh  = d.height-50;

    iWidth  = (4*(iScreenHigh-50))/5;
    iHeight = iScreenHigh-50;

//    panel = new geminiPlotWells( stStruct, stWells, stGrid, 800, 850, lbl, 2);
    panel = new geminiPlotWells( notifier, stStruct, stWells, stGrid,
                                 iScreenWidth-iControlWidth, iScreenHigh-50,
                                 lbl, iGrid );
//    panel = new geminiPlotWells( stStruct, stWells, stGrid, iWidth, iHeight, lbl, 2);

    this.getContentPane().add(pnlTop, BorderLayout.NORTH);
    pnlTop.add(lbl, null);

    this.getContentPane().add(panel,  BorderLayout.CENTER);

    // Display the Frame

//    this.setSize(new Dimension(800, 850));
//    this.setLocation((d.width - this.getSize().width)/2,
//                     (d.height - this.getSize().height)/2);

    this.setSize(iScreenWidth-iControlWidth, iScreenHigh);
    this.setLocation(iControlWidth+1, 1);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** METHOD geminiAreaControlFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class geminiAreaControlFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
	  if (pNotifier != null)
	    pNotifier.notifyObservers(new String("Close Map Area Plot"));

//      close();
//      System.exit(0);
    }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
	pNotifier = null; // Observable Object this Frame
	sTitle    = null;
    stStruct  = null; // Global Common Data
    stGrid    = null; // Land Grid List Data
    stWells   = null; // Well List Data

    // Global Variables

    notifier = null; // Observable Object this Frame

    if (panel != null)
      panel.close();
    panel    = null; // Plot Wells Panel

    // Global Widgets

	dispose();
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Methods Get Minimum & Maximum Latitude & Longitude
   *  @dXMin = Minimum Longitude
   *  @dXMax = Maximum Longitude
   *  @dYMin = Minimum Latitude
   *  @dYMax = Maximum Latitude
   */

  public double getMinLatitude()  { return (panel.getMinLatitude()); }
  public double getMaxLatitude()  { return (panel.getMaxLatitude()); }
  public double getMinLongitude() { return (panel.getMinLongitude()); }
  public double getMaxLongitude() { return (panel.getMaxLongitude()); }

  /** Method getWell()
   *  <p> This method will return the selected well.
   */

  public iqstratHeadersStruct getWell() { return (stWell); }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setWell()
   *  <p> This method will set whether reference well is to be selected or
   *      the edit tops well is to be selected
   *  @param iWell = Well to be selected
   */

  public void setWell( int iWell ) { panel.setWell( iWell ); }

  /** Method setLogType()
   *  <p> This method will set the filter of the wells that are
   *      being plotted.
   *  @param filter = the filter to the Well plot
   */

  public void setLogType(String filter) { panel.setLogType(filter); }

  /** Method setText()
   *  <p> This method will set the filter of the wells that are
   *      being plotted.
   *  @param filter = the filter to the Well plot
   */

  public void setText(int iAdd ) { panel.setText(iAdd); }

  /** Method clearWell()
   *  <p> This method will deselete well
   *  @param iPanel = the Panel cleared
   */

  public void clearWell( int iPanel ) { panel.clearWell(iPanel); }

  /** Method zoom_off()
   *  <p> This method will set the zoom off;
   */

  public void zoom_off() { panel.zoom_off(); }

  /** Method zoom_in()
   *  <p> This method will compute the Minimum & Maximum Latitude & Longitude
   */

  public void zoom_in() { panel.zoom_in(); }

  /** Method zoom_out()
   *  <p> This method will reset the Minimum & Maximum Latitude & Longitude
   */

  public void zoom_out() { panel.zoom_out(); }

  /** Method setField()
   *  <p> This method will set field KID to plot field back ground color
   *  @param sFieldKID = Field KID
   */

  public void setField(String sFieldKID)
  {
	panel.setField(sFieldKID);
	repaint();
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
  }

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
	int    icnt = -1;
    String sArg = new String( (String) arg);

    if (sArg.equals("Well Selected"))
    {
	  stWell = panel.getWell();
      if (pNotifier != null)
        pNotifier.notifyObservers(new String("Well Selected"));
	}

    if (pNotifier != null)
      pNotifier.notifyObservers( new String( "FREE MEMORY" ) );
  }
}


/** CLASS geminiAreaMapFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class geminiAreaMapFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 09/27/2012
 *  @author  John Victorine
 */
