/*
 * @geminiPlotLeases.java Version 1.1 01/24/2014
 *
 * Copyright (c) 2001 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package gemini.plot;

import java.util.Observable;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.Graphics2D.*;
import javax.swing.*;

//import gemini.geminiModifyProjectWellFrame;

import cmn.cmnStruct;

import iqstrat.iqstratHeadersListStruct;         // Well Header List Structure
import iqstrat.iqstratHeadersStruct;             // Well Header Data Structure

import production.plot.productionPlotListStruct; // Production Plot List Structure

import plot.plotWellStatusStruct;                // Plot Well Status Data Structure
import plot.plotMapping;                         // Plot Well Symbols

import util.utilLandGridListStruct;              // TRS Grid Lines List Structure
import util.utilPlotLandGridImage;               // TRS Grid Image

/** Class geminiPlotLeases
 *  <p> This class will create display the Wells
 *
 *  @version 1.1 01/24/2014
 *  @author  John R. Victorine
 */

public class geminiPlotLeases extends Canvas
    implements MouseListener, MouseMotionListener
{
  private static final int COUNTY_GRID = util.utilPlotLandGrid.COUNTY_GRID;
  private static final int AREA_GRID   = util.utilPlotLandGrid.AREA_GRID;
  private static final int NO_TEXT     = util.utilPlotLandGrid.NO_TEXT;

  private static final int ZOOM_OFF    = 0;
  private static final int ZOOM_OUT    = 1;
  private static final int ZOOM_IN     = 2;

  private int iAction = ZOOM_OFF;

  public static final int STOP     = 0;
  public static final int PAUSE    = 1;
  public static final int RUN      = 2;

  private int iPlay   = STOP;

  // Input Variables

  private Observable               notifier   = null; // Observable Object this Frame
  private cmnStruct                stStruct   = null; // Common Data Structure
  private iqstratHeadersListStruct stPlot     = null; // Well Header List Data Structure
  private utilLandGridListStruct   stLandGrid = null; // TRS Grid Lines Data List Structure
  private JLabel                   lbl        = null; // Label to display mouse location
  private int                      iGrid      = NO_TEXT;

  private productionPlotListStruct stData     = null; // Production Plot List Data Structure
  private int                      iStartYear = 0;    // Starting Year of Production Data
  private int                      iEndYear   = 0;    // Ending Year of Production Data
  private double                   dMinCum    = 0.0;  // Minimum Cumulative value
  private double                   dMaxCum    = 0.0;  // Maximum Cumulative value
  private double                   dMedium    = 0.0;  // Medium Value of the Cumlative
  private double                   dMean      = 0.0;  // Mean (Average) Value of the Cumulative
  private double                   dMinProd   = 0.0;  // Minimum Production value
  private double                   dMaxProd   = 0.0;  // Maximum Production value

  private int                      iYear      = 0;    // Year date being plotted

  // Global Variables

  private int iRows         = 0;
  private int iX[]          = null;
  private int iY[]          = null;
  private int iSymbol[]     = null;

  private String sFieldKID  = "0";

  // Well Select Number within well list

  private int iWell         = 0;
  private int iSelectedWell = -1;
  private int iWells[]      = { -1, -1, -1, -1 };  // Wells Selected\

  public static final int _TOTAL_WELLS = gemini.gui.geminiAreaControlFrame._TOTAL_WELLS;

  // -- Filter Wells By

  private String               sLogType  = new String("ALL");

  // -- Show Text

  private static final int _SYMBOL     = 0;
  private static final int _WELL_NAME  = 1;
  private static final int _API_NUMBER = 2;

  private int                  iShow     = _SYMBOL;

  // Plot Variables

  private int                  iWidth    = 0;
  private int                  iHeight   = 0;
  private int                  iXBegin   = 0;
  private int                  iYBegin   = 0;

  private double               dXMaxOrig = 0.0;
  private double               dXMinOrig = 0.0;
  private double               dYMaxOrig = 0.0;
  private double               dYMinOrig = 0.0;

  private double               dXMax     = 0.0;
  private double               dXMin     = 0.0;
  private double               dYMax     = 0.0;
  private double               dYMin     = 0.0;
  private plotMapping          plotMap   = new plotMapping();
  private plotWellStatusStruct stSymbols = new plotWellStatusStruct();

//  private int                  iXGrid[]  = null;
//  private int                  iYGrid[]  = null;

  private Image                imageLandGrid = null;
  private int                  IN        = 0;

  private int                  iAddBox   = 0;
  private int                  iAddStart = 0;
  private int                  iAddWidth = 0;

  private int                  iXStart   = 0;
  private int                  iXEnd     = 0;
  private int                  iYStart   = 0;
  private int                  iYEnd     = 0;
/*
  private static final int _TOTAL      = 6;
  private static final int _COLORS[][] = {
	{ 255,   0,   0 }, // RED
    { 255, 165,   0 }, // orange
    { 255, 255,   0 }, // yellow
    {   0, 255,   0 }, // green
//    {   0,   0, 255 }, // blue
    { 135, 206, 235 }, // sky blue
    { 140,   0, 211 }}; // dark violet
*/
  private static final int _TOTAL      = 40;
  private static final int _COLORS[][] = {
	{ 255,   0,   0 }, // RED
    { 102,  51,   0 }, // dark brown
    { 165,  42,  42 }, // brown
    { 160,  82,  45 }, // sienna
    { 222, 184, 135 }, // burlywood
    { 210, 180, 140 }, // tan
    { 250, 128, 114 }, // salmon
    { 255,  69,   0 }, // orange red 1
    { 255, 140,   0 }, // dark orange
    { 255, 165,   0 }, // orange

    { 255, 255, 128 }, // middle yellow
    { 255, 225,  25 }, // dark yellow
    { 255, 255,   0 }, // yellow
    { 173, 255,  47 }, // green yellow
    { 154, 205,  50 }, // yellow green
    { 154, 255, 154 }, // pale green 1
    { 128, 255, 128 }, // light green
    {   0, 255,   0 }, // green
    { 124, 252,   0 }, // lawn green
    {  34, 139,  34 }, // forest green

    { 107, 142,  35 }, // olive drab
    {  64, 224, 208 }, // turguoise
    { 127, 255, 212 }, // aquamarine 1
    { 102, 205, 170 }, // aquamarine 3
    {  69, 139, 116 }, // aquamarine 4
    { 128, 255, 255 }, // middle cyan
    {   0, 255, 255 }, // cyan
    { 135, 206, 235 }, // sky blue
    {   0, 191, 255 }, // deep sky blue 1
    {   0, 178, 238 }, // deep sky blue 2

    { 128, 128, 255 }, // middle blue
    {  65, 105, 225 }, // royal blue
    {   0,   0, 255 }, // blue
    { 255, 182, 193 }, // light pink
    { 255, 105, 180 }, // hot pink
    { 255,  20, 147 }, // deep pink
    { 255, 128, 255 }, // light magenta
    { 255,   0, 255 }, // magenta
    { 238, 130, 238 }, // violet
    { 140,   0, 211 }, // dark violet
  };

  /** Constructor geminiPlotLeases()
   *  <p> This is the Constructor for this class.
   *  @param stStruct - Plot Data Structure.
   *  @param st       - Well Information Data List Structure
   *  @param stL      - Land Grid Data List Structure
   *  @param width    - Width of the Well Map
   *  @param height   - Height of the Well Map
   *  @param lbl      - Text Field to display Well Information.
   *  @param iGrid    - Plot Text with Grid or NOT.
   */

  public geminiPlotLeases( Observable notifier,
                           cmnStruct  stStruct,
                           iqstratHeadersListStruct st,
                           utilLandGridListStruct   stL,
                           int        width,
                           int        height,
                           JLabel     lbl,
                           int        iGrid )
  {
    try
    {
	  this.notifier   = notifier;
      this.stStruct   = stStruct;
      this.lbl        = lbl;

      this.stPlot     = st;
      this.iRows      = stPlot.iCount;

      this.stLandGrid = stL;
      this.iGrid      = iGrid;

      this.iXBegin    = 0;
      this.iYBegin    = 0;
      this.iWidth     = width -iXBegin;
      this.iHeight    = height-iYBegin;

      jbInit();
    }
    catch (Exception e)
    {
      JOptionPane.showMessageDialog((Component) null,
                                    "geminiPlotLeases(): " + e.getMessage(),
                                    "ERROR",
                                    JOptionPane.ERROR_MESSAGE);
    }
  }

  /** Method jbInit()
   *  <p> This method will create the Linear Plot.
   *  @throws Exception
   */

  public void jbInit() throws Exception
  {
    setSize(iWidth, iHeight);
    setBackground(new Color(250, 250, 250));

    iX      = new int[iRows];
    iY      = new int[iRows];
    iSymbol = new int[iRows];

    getMinMaxPosition();
    imageLandGrid = utilPlotLandGridImage.makePlotImage( stLandGrid,
                                                         iWidth,
                                                         iHeight,
                                                         dXMin, dXMax,
                                                         dYMin, dYMax,
                                                         iGrid );
    getData();
    getWellSymbol();

    addMouseListener(this);
    addMouseMotionListener(this);
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    notifier   = null; // Observable Object this Frame
    stStruct   = null;
    stPlot     = null;
    stLandGrid = null;
    lbl        = null;

    // Global Variables

    iX         = null;
    iY         = null;
    iSymbol    = null;

    stData     = null;

    // Well Select Number within well list

    iWells     = null;

    // -- Filter Wells By

    sLogType   = null;

    // Plot Variables

    if (plotMap != null)
      plotMap.close();
    plotMap = null;

//    iXGrid        = null;
//    iYGrid        = null;

    imageLandGrid = null;
  }

  /* ======================================================================== *
   * ------------------------- GET Methods ---------------------------------- *
   * ======================================================================== */

  /** Methods Get Minimum & Maximum Latitude & Longitude
   *  @dXMin = Minimum Longitude
   *  @dXMax = Maximum Longitude
   *  @dYMin = Minimum Latitude
   *  @dYMax = Maximum Latitude
   */

  public double getMinLatitude()  { return (dYMin); }
  public double getMaxLatitude()  { return (dYMax); }
  public double getMinLongitude() { return (dXMin); }
  public double getMaxLongitude() { return (dXMax); }

  /** Method getLeaseID()
   *  <p> This method will retrieve the Lease Selected
   */

  public int getLeaseID() { return (iSelectedWell); }

  /** Method getWell()
   *  <p> This method will return the selected well.
   */

  public iqstratHeadersStruct getWell()
  {
	iqstratHeadersStruct st = null;

	if (stPlot != null)
	{
	  if (iSelectedWell > -1)
      {
		st = stPlot.stItem[iSelectedWell];
      }
	}

	return (st);
  }

  /** Method getMinMaxPosition()
   *  <p> This method will determine the minimum and maximum position of the
   *      wells that have been passed in.
   */

  private void getMinMaxPosition()
  {
    int    i          = 0;
    int    j          = 0;
    int    iTemp      = 0;
    int    iFound     = 0;
    int    iStart     = 0;
    double dLongitude = 0.0;
    double dLatitude  = 0.0;
    double dMinLat    = 0.0;
    double dMaxLat    = 0.0;
    double dMinLong   = 0.0;
    double dMaxLong   = 0.0;

    double delX       = 0.0;
    double delY       = 0.0;

    double dTemp      = 0.0;
    double dLat       = 0.0;
    double dLong      = 0.0;

    for (i=0; i<stLandGrid.iCount; i++)
    {
      for (j=0; j<stLandGrid.stItem[i].iTotal; j++)
      {
        if (iStart == 0)
        {
          dMinLat  = stLandGrid.stItem[i].dLatitude[j];
          dMaxLat  = stLandGrid.stItem[i].dLatitude[j];
          dMinLong = stLandGrid.stItem[i].dLongitude[j];
          dMaxLong = stLandGrid.stItem[i].dLongitude[j];
          iStart   = 1;
        }

        if (dMinLat > stLandGrid.stItem[i].dLatitude[j])
          dMinLat  = stLandGrid.stItem[i].dLatitude[j];

        if (dMaxLat < stLandGrid.stItem[i].dLatitude[j])
          dMaxLat  = stLandGrid.stItem[i].dLatitude[j];

        if (dMinLong > stLandGrid.stItem[i].dLongitude[j])
          dMinLong  = stLandGrid.stItem[i].dLongitude[j];

        if (dMaxLong < stLandGrid.stItem[i].dLongitude[j])
          dMaxLong  = stLandGrid.stItem[i].dLongitude[j];
      }
    }

    for (i=0; i<iRows; i++)
    {
      dLatitude  = stPlot.stItem[i].dLatitude;
      dLongitude = stPlot.stItem[i].dLongitude;

      if ((Math.abs(dLongitude) > 0.0) &&
          (Math.abs(dLatitude)  > 0.0))
      {
        if (iFound == 0)
        {
          dXMin = dLongitude;
          dXMax = dXMin;

          dYMin = dLatitude;
          dYMax = dYMin;

          iFound = 1;
        }

        if (dLongitude < dXMin)
          dXMin = dLongitude;

        if (dLongitude > dXMax)
          dXMax = dLongitude;

        if (dLatitude < dYMin)
          dYMin = dLatitude;

        if (dLatitude > dYMax)
          dYMax =dLatitude;
      }
    }

    if (dXMin < dMinLong)
      dXMin = dMinLong;

    if (dXMax > dMaxLong)
      dXMax = dMaxLong;

    if (dYMin < dMinLat)
      dYMin = dMinLat;

    if (dXMax > dMinLat)
      dYMax = dMaxLat;

    if (IN == 0)
    {
      dXMin = dMinLong;
      dXMax = dMaxLong;
      dYMin = dMinLat;
      dYMax = dMaxLat;
    }

    double dX = 0.05 * (dXMax - dXMin);
    double dY = 0.05 * (dYMax - dYMin);

    dXMin = dXMin - dX;
    dXMax = dXMax + dX;
    dYMin = dYMin - dY;
    dYMax = dYMax + dY;

    delX  = Math.abs(dXMax - dXMin);
    delY  = Math.abs(dYMax - dYMin);

    dXMinOrig = dXMin;
    dXMaxOrig = dXMax;
    dYMinOrig = dYMin;
    dYMaxOrig = dYMax;

    dTemp = iWidth / (4*iHeight/5);
  }

  /** Method getData()
   *  <p> This method will convert the raw data to pixels so the data can be
   *      plotted.
   */

  public void getData()
  {
    int    iRatio;
    int    i,j,k;
    double dLongitude = 0.0;
    double dLatitude  = 0.0;
    int    iCont      = 0;


    for (i=0; i<iRows; i++)
    {
      dLongitude = stPlot.stItem[i].dLongitude;
      iRatio = (int)((double)iWidth * (dLongitude-dXMin) / (dXMax - dXMin));
      iX[i] = iXBegin + iRatio;
    }

    for (i=0; i<iRows; i++)
    {
      dLatitude = stPlot.stItem[i].dLatitude;
      iRatio = (int)((double)iHeight * (dLatitude - dYMin)/ (dYMax - dYMin));
      iY[i] = iYBegin + iHeight - iRatio;
    }
  }

  /** Method getWellSymbol()
   *  <p> This method will convert the Well Symbol to one of the recognized
   *      symbols.
   */

  private void getWellSymbol()
  {
    for (int i=0; i<iRows; i++)
    {
      iSymbol[i] = plot.plotWellStatusStruct.NONE;

      for (int j=0; j<stSymbols.iCount; j++)
      {
        if (stPlot.stItem[i].status.equals(stSymbols.status[j][0]))
          iSymbol[i] = stSymbols.iStatus[j];
      }
    }
  }

  /* ======================================================================== *
   * ------------------------- SET Methods ---------------------------------- *
   * ======================================================================== */

  /** Method clearWell()
   *  <p> This method will deselete well
   *  @param iPanel = the Panel cleared
   */

  public void clearWell( int iPanel )
  {
	iWells[iPanel] = -1;
	repaint();
  }

  /** Method setWell()
   *  <p> This method will set whether reference well is to be selected or
   *      the edit tops well is to be selected
   *  @param iWell = Well to be selected
   */

  public void setWell( int iWell ) { this.iWell = iWell; }

  /** Method setLogType()
   *  <p> This method will set the filter of the wells that are
   *      being plotted.
   *  @param filter = the filter the Well plot
   */

  public void setLogType(String filter)
  {
    sLogType = new String(filter);
    repaint();
  }

  /** Method setText()
   *  <p> This method will set the filter of the wells that are
   *      being plotted.
   *  @param filter = the filter to the Well plot
   */

  public void setText(int iAdd )
  {
	iShow = iAdd;
	repaint();
  }

  /** Method setWellList()
   *  <p> This method will refresh the list of wells to be plotted.
   *  @param st - Well Position List Data Structure.
   *  @param iGrid   = Plot Text with Grid or NOT.
   */


  public void setWellList(iqstratHeadersListStruct st, int iGrid)
  {
    stPlot  = st;
    iRows   = stPlot.iCount;

    iX      = new int[iRows];
    iY      = new int[iRows];
    iSymbol = new int[iRows];

    this.iGrid = iGrid;

    getMinMaxPosition();
    imageLandGrid = utilPlotLandGridImage.makePlotImage(stLandGrid,
                                                        iWidth,
                                                        iHeight,
                                                        dXMin, dXMax,
                                                        dYMin, dYMax,
                                                        iGrid);
    getData();
    getWellSymbol();
//System.out.println("WELL LIST "+dXMin+" "+dXMax+" - "+dYMin+" "+dYMax);

    repaint();
  }

  /** Method setProductionData()
   *  <p> This method will set the Production Plot Data
   *  @param stData = the production plot data List Structure
   */

  public void setProductionData( productionPlotListStruct stData,
                                 int iStartYear,  int iEndYear,
                                 double dMinCum,  double dMaxCum,
                                 double dMinProd, double dMaxProd,
                                 double dMedium,  double dMean)
  {
	this.stData     = stData;
	this.iStartYear = iStartYear;
	this.iEndYear   = iEndYear;
	this.dMinCum    = dMinCum;
	this.dMaxCum    = dMaxCum;
	this.dMinProd   = dMinProd;
	this.dMaxProd   = dMaxProd;
	this.dMedium    = dMedium;
	this.dMean      = dMean;

//System.out.println("PRODUCTION: " +dXMin+" "+dXMax+" - "+dYMin+" "+dYMax);
  }

  /** Method setField()
   *  <p> This method will set field KID to plot field back ground color
   *  @param sFieldKID = Field KID
   */

  public void setField(String sFieldKID)
  {
	this.sFieldKID = new String(sFieldKID);
    repaint();
  }

  /** Method compute()
   *  <p> This method will re-compute the Minimum & Maximum Latitude & Longitude
   */

  private void compute()
  {
    double dX = 0.0;
    double dY = 0.0;

    double dX0 = 0.0;
    double dX1 = 0.0;
    double dY0 = 0.0;
    double dY1 = 0.0;

//System.out.println("A "+dXMin+" "+dXMax+" - "+dYMin+" "+dYMax);
    dX0   = dXMin + (dXMax-dXMin) * (iXStart-iXBegin) / iWidth;
    dX1   = dXMin + (dXMax-dXMin) * (iXEnd-iXBegin) / iWidth;

    dY1   = dYMax - (dYMax-dYMin) * (iYStart-iYBegin) / iHeight;
    dY0   = dYMax - (dYMax-dYMin) * (iYEnd-iYBegin) / iHeight;
//System.out.println("B "+dX0+" "+dX1+" - "+dY0+" "+dY1);

    dXMin = dX0;
    dXMax = dX1;
    dYMin = dY0;
    dYMax = dY1;

    dX    = 0.05 * (dXMax - dXMin);
    dY    = 0.05 * (dYMax - dYMin);

    dXMin = dXMin - dX;
    dXMax = dXMax + dX;
    dYMin = dYMin - dY;
    dYMax = dYMax + dY;

    getData();
    imageLandGrid = utilPlotLandGridImage.makePlotImage( stLandGrid,
                                                         iWidth,
                                                         iHeight,
                                                         dXMin, dXMax,
                                                         dYMin, dYMax,
                                                         iGrid );

    repaint();
  }

  /** Method zoom_off()
   *  <p> This method will set the zoom off;
   */

  public void zoom_off() { iAction = ZOOM_OFF; }

  /** Method zoom_in()
   *  <p> This method will compute the Minimum & Maximum Latitude & Longitude
   */

  public void zoom_in() { iAction = ZOOM_IN; }

  /** Method zoom_out()
   *  <p> This method will reset the Minimum & Maximum Latitude & Longitude
   */

  public void zoom_out()
  {
    dXMin   = dXMinOrig;
    dXMax   = dXMaxOrig;
    dYMin   = dYMinOrig;
    dYMax   = dYMaxOrig;

    getData();
    imageLandGrid = utilPlotLandGridImage.makePlotImage( stLandGrid,
                                                         iWidth,
                                                         iHeight,
                                                         dXMin, dXMax,
                                                         dYMin, dYMax,
                                                         iGrid );

    iAction = ZOOM_OFF;

    repaint();
  }

  /** Method setArea()
   *  <p> This method will zoom into the area
   */

  public void setArea( double dXMin, double dXMax, double dYMin, double dYMax )
  {
    this.dXMin = dXMin;
    this.dXMax = dXMax;
    this.dYMin = dYMin;
    this.dYMax = dYMax;

//    dXMinOrig  = dXMin;
//    dXMaxOrig  = dXMax;
//    dYMinOrig  = dYMin;
//    dYMaxOrig  = dYMax;
//System.out.println("AREA "+this.dXMin+" "+this.dXMax+" - "+this.dYMin+" "+this.dYMax);

    getData();
    imageLandGrid = utilPlotLandGridImage.makePlotImage( stLandGrid,
                                                         iWidth,
                                                         iHeight,
                                                         this.dXMin, this.dXMax,
                                                         this.dYMin, this.dYMax,
                                                         iGrid );

    iAction = ZOOM_OFF;

    repaint();
  }

  /** Method setPlay()
   *  <p> This method will set the play action
   */

  public void setPlay(int iPlay)
  {
	this.iPlay = iPlay;
    repaint();
  }

  /** Method setPlay()
   *  <p> This method will set the play action
   */

  public void setYear(int iYear)
  {
	this.iYear = iYear;
	repaint();
  }

  /* ======================================================================== *
   *  MouseListener Methods required when implementing the MouseListener Class.
   * ======================================================================== */

  /** Method mouseMoved()
   *  <p> This method will retrieve the location of the mouse pointer and
   *      convert the postion to latitude and longitude position. It will then
   *      display the text in the Volumetric Panel.
   * @param event - Mouse Moved Event
   */

  public void mouseMoved(MouseEvent event)
  {
    int    iXm       = event.getX();
    int    iYm       = event.getY();
    int    iTemp     = 0;
    int    iFound    = 0;
    double dTemp     = 0.0;
    double dX        = 0.0;
    double dY        = 0.0;
    String sPosition = new String("");
    String sLAS      = ""; // Well Contains LAS File Data          (L)
    String sTYPE     = ""; // Well Contains LAS File Data (L)

    if ((stPlot != null) && (stData != null))
    {
      if ((iXm > iXBegin) && (iXm < iXBegin + iWidth) &&
          (iYm > iYBegin) && (iYm < iYBegin + iHeight))
      {
        dX = dXMin + (dXMax - dXMin) * (double) (iXm - iXBegin)/(double) (iWidth);
        dTemp = dX * 1000.0;
        iTemp = (int) dTemp;
        dX = (double) iTemp / 1000.0;

        dY = dYMin - (dYMax - dYMin) * (double) (iYm - (iYBegin+iHeight)) /
            (double) (iHeight);
        dTemp = dY * 1000.0;
        iTemp = (int) dTemp;
        dY = (double) iTemp / 1000.0;

        iFound = 0;
        for (int i=0; i<iRows; i++)
        {
          sLAS = ""; // Well Contains LAS File Data (L)

          if ((iXm > (iX[i]-5)) && (iXm < (iX[i]+5)) &&
              (iYm > (iY[i]-5)) && (iYm < (iY[i]+5)))
          {
            if (stPlot.stItem[i].iLAS > 0)       { sLAS = new String("L"); }
            if (stPlot.stItem[i].ilitho_log > 0) { sLAS = new String("LITHO"); }
            if (stPlot.stItem[i].iGeo > 0)       { sLAS = new String("GEO"); }

            sPosition = new String(stPlot.stItem[i].sAPI   + "  " +
                                   stPlot.stItem[i].sName  +
                                   " Latitude: "  + stPlot.stItem[i].dLatitude +
                                   " Longitude: " + stPlot.stItem[i].dLongitude);

            if (sLogType.equals("ALL") || sLogType.equals(sLAS))
            {
              iFound = 1;
            }

            if (lbl != null)
              lbl.setForeground(Color.red);
          }
        }

        if (iFound == 0)
        {
          sPosition = new String("Latitude: " + dY + " Longitude: " + dX);
          if (lbl != null)
            lbl.setForeground(Color.black);
        }

        if (lbl != null)
          lbl.setText(sPosition);
      }
	}
  }

  /** Method mouseDragged()
   *  <p> This method will dragg the mouse and draw the Rubber Band.
   * @param event - Mouse Moved Event
   */

  public void mouseDragged(MouseEvent event)
  {
    if ((stPlot != null) && (stData != null))
    {
      iAddBox = 1;
      showRubberBand(event);
	}
  }

  public void mouseReleased(MouseEvent event)
  {
    iXEnd = event.getX();
    iYEnd = event.getY();

    if ((stPlot != null) && (stData != null))
    {
      showRubberBand(event);
      iAddBox = 0;

      if (iAction == ZOOM_IN)
        compute();
	}
  }

  /** Method mousePressed(MouseEvent)
   *  <p> gets the current coordinates and creates a new rectangle,
   *      updates the rectangle to be drawn and
   *      calls repaint on the parent panel
   *  @param event - MouseEvent info
   */

  public void mousePressed(MouseEvent event)
  {
    if ((stPlot != null) && (stData != null))
    {
      iXStart = event.getX();
      iXEnd   = 0;
      iYStart = event.getY();
      iYEnd   = 0;
      iAddBox = 1;
      showRubberBand(event);
    }
  }

  //public void mousePressed(MouseEvent event)  {}
  public void mouseEntered(MouseEvent event)  {}
  public void mouseExited(MouseEvent event)   {}

  /** Method mouseClicked(MouseEvent)
   *  <p> user clicked mouse get Well
   *  @param event - MouseEvent info
   */

  public void mouseClicked(MouseEvent event)
  {
    int    iXm    = event.getX();
    int    iYm    = event.getY();
    int    iFound = 0;
    String sURL   = "";
    String sLAS   = ""; // Well Contains LAS File Data (L)
    String sTYPE  = ""; // Well Contains LAS File Data (L)

    iXStart = event.getX();
    iYStart = event.getY();

    iFound = 0;
    if ((stPlot != null) && (stData != null))
    {
      for (int i=0; i<iRows; i++)
      {
        sLAS = ""; // Well Contains LAS File Data (L)

        if ((iXm > (iX[i]-5)) && (iXm < (iX[i]+5)) &&
            (iYm > (iY[i]-5)) && (iYm < (iY[i]+5)))
        {
          if ((sLogType.equals("L")) && (stPlot.stItem[i].iLAS > 0))
          {
  		    sLAS = new String("L");
		  }
          if ((sLogType.equals("LITHO")) && (stPlot.stItem[i].ilitho_log > 0))
          {
		    sLAS = new String("LITHO");
		  }
          if ((sLogType.equals("GEO")) && (stPlot.stItem[i].iGeo > 0))
          {
		    sLAS = new String("GEO");
		  }

          if ((iFound == 0) &&
              (sLogType.equals("ALL") || sLogType.equals(sLAS)))
          {
            iFound = 1;
          }

          if (iFound == 1)
          {
            if (stPlot.stItem[i] != null)
            {
		  	  iSelectedWell = i;
		  	  iWells[iWell] = i;
              repaint();

  	          if (notifier != null)
	            notifier.notifyObservers(new String("Well Selected"));
			}
          }
        }
      }
    }
  }

  /** Method showRubberBand(MouseEvent)
   *  <p> This method is adapted from java.sun.com(MouseMotionEventDemo.java)
   *      updates the size of the current rectangle
   *  @param e - MouseEvent info
   */

  public void showRubberBand(MouseEvent e)
  {
    iXEnd = e.getX();
    iYEnd = e.getY();

    this.repaint();
  }

  /* ======================================================================== *
   *  Drawing Methods                                                         *
   * ======================================================================== */

  /** Method drawRubberBand()
   *  <p> This method will draw a box around the selected area.
   *  @param g - Graphics pointer.
   */

  public void drawRubberBand(Graphics g)
  {
    int iXStartMouse = iXStart;
    int iYStartMouse = iYStart;
    int iMouseWidth  = 0;
    int iMouseHeight = 0;

    if (iXEnd < iXStart)
    {
      iXStartMouse = iXEnd;
      iMouseWidth  = iXStart - iXEnd;
    }
    else
      iMouseWidth  = iXEnd   - iXStart;

    if (iYEnd < iYStart)
    {
      iYStartMouse = iYEnd;
      iMouseHeight = iYStart - iYEnd;
    }
    else
      iMouseHeight = iYEnd   - iYStart;

    if (iAddBox == 1)
    {
      g.setColor(Color.cyan);
      g.drawRect(iXStartMouse, iYStartMouse, iMouseWidth, iMouseHeight);

      g.setColor(Color.gray);
      g.drawRect(iXStartMouse+1, iYStartMouse+1, iMouseWidth, iMouseHeight);
    }
  }

  /** Method drawWells()
   *  <p> This method will draw the Linear Plot Curves.
   *  @param g - Graphics pointer.
   */

  public void drawWells(Graphics g)
  {
    int i, j;
    int    iSelect = 0;
    int    iLevel  = -1;
    String sLAS    = ""; // Well Contains LAS File Data (L)
    String sTYPE   = ""; // Well Contains LAS File Data (L)
    String sName   = ""; // Text to add to each well
    int    iCont   = 0;

    Font        fsb  = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm = g.getFontMetrics(fsb);

    g.setFont( fsb );
    g.setColor(Color.black);

    for (i=0; i<iRows; i++)
    {
	  iCont   = 0;
	  iSelect = 0;
	  for (j=0; j<_TOTAL_WELLS; j++)
	  {
	    if (i == iWells[j])
	    {
	      iSelect = 1;
		}
	  }

      sName = "";
      if ((iShow == _WELL_NAME) || (iShow == _API_NUMBER))
      {
		if (iShow == _WELL_NAME)
		  sName = stPlot.stItem[i].sName;
		else
		  sName = stPlot.stItem[i].sAPI;
	  }

      sLAS = ""; // Well Contains LAS File Data (L)

      if ((iX[i] >= iXBegin) && (iX[i] <= iXBegin+iWidth) &&
          (iY[i] >= iYBegin) && (iY[i] <= iYBegin+iHeight))
      {
//        iLevel = -1;
//        if (stPlot.stItem[i].iTops > 0)
//        {
//          iLevel = plot.plotMapping._POOR;
//	    }

        if ((sLogType.equals("L")) && (stPlot.stItem[i].iLAS > 0))
        {
		  sLAS  = new String("L");
		}

        if ((sLogType.equals("LITHO")) && (stPlot.stItem[i].ilitho_log > 0))
        {
		  sLAS = new String("LITHO");
		}

        if ((sLogType.equals("GEO")) && (stPlot.stItem[i].iGeo > 0))
        {
		  sLAS = new String("GEO");
		}

//        if (iCont == 1)
        {
		  g.setColor(Color.black);
		  g.drawString(sName, iX[i]+9, iY[i]);

          plotMap.drawWellSymbol(g, iSymbol[i], iX[i], iY[i]);
          plotMap.drawHalo(g, iLevel, iX[i], iY[i]);
          if (iSelect == 1)
            plotMap.drawHalo(g, plot.plotMapping._SELECTED, iX[i], iY[i]);
		}
      }
    }
  }

  /** Method drawProduction()
   *  <p> This method will draw the Production Bubble.
   *  @param g - Graphics pointer.
   */

  public void drawProduction(Graphics g)
  {
	int    iCont    = 0;
	int    iRadius  = 0;
	int    iRed     = 0;
	int    iGreen   = 0;
	int    iBlue    = 0;
	double dMinimum = 0.0;
	double dMaximum = 0.0;
	double data     = 0.0;
	double dRatio   = 200.0;
	int    iColor   = 0;
	int    iP       = 0;
	float  fRed     = 0;
	float  fGreen   = 0;
	float  fBlue    = 0;

    Font        fsb  = new Font("Serif", Font.BOLD, 11);
    FontMetrics fsbm = g.getFontMetrics(fsb);

    g.setFont( fsb );

    dMinimum = dMinCum;
    dMaximum = dMean + 0.2 * dMaxCum;

    if (iPlay != STOP)
    {
      for (int i=0; i<iRows; i++)
      {
		if (stData.stItem[i] != null)
		{
          for (int j=0; j<stData.stItem[i].iRows; j++)
          {
  	        iCont   = 0;

            if ((iX[i] >= iXBegin) && (iX[i] <= iXBegin+iWidth) &&
                (iY[i] >= iYBegin) && (iY[i] <= iYBegin+iHeight))
            {
              if (stData.stItem[i].iTime[j] == iYear)
              {
			    iP    = j;
	            iCont = 1;
	          }
	          else if ((j==stData.stItem[i].iRows-1) &&
	                   (stData.stItem[i].iTime[j] <= iYear))
	          {
			    iP    = j;
			    iCont = 2;
		      }

              if (iCont > 0)
              {
                data = stData.stItem[i].dCumulative[iP];
                if (data > dMaximum) { data = dMaximum; }

                iRadius = (int) (dRatio * (data - dMinimum) / (dMaximum - dMinimum));
                if (iRadius < 10) iRadius = 10;
                iColor  = (int) ((_TOTAL * data) / dMaximum);

                if (iColor >= _TOTAL) iColor = _TOTAL-1;

                iRed   = 200;
                iGreen = 200;
                iBlue  = 200;

                if (iCont == 1)
                {
                  iRed   = _COLORS[iColor][0];
                  iGreen = _COLORS[iColor][1];
                  iBlue  = _COLORS[iColor][2];
			    }

                fRed   = (float) iRed / (float) 255.0;
                fGreen = (float) iGreen / (float) 255.0;
                fBlue  = (float) iBlue / (float) 255.0;

  		        g.setColor(new Color(fRed, fGreen, fBlue, .5f));
//  	  	       g.setColor(new Color(iRed, iGreen, iBlue));
	            g.fillOval(iX[i]-iRadius/2, iY[i]-iRadius/2, iRadius, iRadius);

	            g.setColor(Color.black);
	            g.drawString(""+stData.stItem[i].iSources[iP], iX[i]-2, iY[i]+4);
		  	  }
		    }
	      }
		}
	  }
	}
  }

  /** Method drawLegend()
   *  <p> This method will draw the legend for the production data.
   *  @param g - Graphics pointer.
   */

  public void drawLegend(Graphics g)
  {
//	double dMinimum = 0.0;
//	double dMaximum = 0.0;
	float  fRed     = 0;
	float  fGreen   = 0;
	float  fBlue    = 0;
	int    iColor   = 0;
	int    iRows    = 1000;
	int    iRadius  = 200;
	int    iRatio   = 0;

    Font        fsb  = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm = g.getFontMetrics(fsb);

    g.setFont( fsb );

//    dMinimum = dMinCum;
//    dMaximum = dMean + 0.2 * dMaxCum;

    if (iPlay != STOP)
    {
      for (int i=0; i<iRadius; i++)
      {
        iColor  = (_TOTAL * i) / iRadius;

        fRed   = (float) _COLORS[iColor][0] / (float) 255.0;
        fGreen = (float) _COLORS[iColor][1] / (float) 255.0;
        fBlue  = (float) _COLORS[iColor][2] / (float) 255.0;
        g.setColor(new Color(fRed, fGreen, fBlue, .5f));

        g.fillRect(i+60, 0, 1, 10);
	  }

      g.setColor(Color.black);
      g.drawString("Minimum", 0, 10);
      g.drawString("Mean + Maximum / 5", 270, 10);
	}
  }

  /** Method paint()
   *  <p> This method will create the Linear Plot.
   *  @param g - Graphics pointer.
   */

  public void paint(Graphics g)
  {
	int   iYears  = 1;
	int   iYear   = 0;

    g.drawImage(imageLandGrid, 0, 0, iWidth, iHeight, null);
    drawLegend(g);
    drawProduction(g);
    if (iPlay == STOP)
      drawWells(g);
    if (iAction == ZOOM_IN)
      drawRubberBand(this.getGraphics());
  }
}
