/*
 * @geminiPlotWells.java Version 1.1 12/26/2002
 *
 * Copyright (c) 2001 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package gemini.plot;

import java.util.Observable;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.Graphics2D.*;
import javax.swing.*;

//import gemini.geminiModifyProjectWellFrame;

import cmn.cmnStruct;

import iqstrat.iqstratHeadersListStruct;
import iqstrat.iqstratHeadersStruct;

import plot.plotWellStatusStruct;
import plot.plotMapping;

import util.utilLandGridListStruct;
import util.utilPlotLandGridImage;

/** Class geminiPlotWells
 *  <p> This class will create display the Wells
 *
 *  @version 1.1 12/26/2002
 *  @author  John R. Victorine
 */

public class geminiPlotWells extends Canvas
    implements MouseListener, MouseMotionListener
{
  private static final int COUNTY_GRID = util.utilPlotLandGrid.COUNTY_GRID;
  private static final int AREA_GRID   = util.utilPlotLandGrid.AREA_GRID;
  private static final int NO_TEXT     = util.utilPlotLandGrid.NO_TEXT;

  private static final int ZOOM_OFF    = 0;
  private static final int ZOOM_OUT    = 1;
  private static final int ZOOM_IN     = 2;

  private int iAction = ZOOM_OFF;

  // Input Variables

  private Observable               notifier   = null; // Observable Object this Frame
  private cmnStruct                stStruct   = null;
  private iqstratHeadersListStruct stPlot     = null;
  private utilLandGridListStruct   stLandGrid = null;
  private JLabel                   lbl        = null;
  private int                      iGrid      = NO_TEXT;

  // Global Variables

  private int iRows         = 0;
  private int iX[]          = null;
  private int iY[]          = null;
  private int iSymbol[]     = null;

  private String sFieldKID  = "0";

  // Well Select Number within well list

  private int iWell         = 0;
  private int iSelectedWell = -1;
  private int iWells[]      = { -1, -1, -1, -1 };  // Wells Selected\

  public static final int _TOTAL_WELLS = gemini.gui.geminiAreaControlFrame._TOTAL_WELLS;

  // -- Filter Wells By

  private String               sLogType  = new String("ALL");

  // -- Show Text

  private static final int _SYMBOL     = 0;
  private static final int _WELL_NAME  = 1;
  private static final int _API_NUMBER = 2;

  private int                  iShow     = _SYMBOL;

  // Plot Variables

  private int                  iWidth    = 0;
  private int                  iHeight   = 0;
  private int                  iXBegin   = 0;
  private int                  iYBegin   = 0;

  private double               dXMaxOrig = 0.0;
  private double               dXMinOrig = 0.0;
  private double               dYMaxOrig = 0.0;
  private double               dYMinOrig = 0.0;

  private double               dXMax     = 0.0;
  private double               dXMin     = 0.0;
  private double               dYMax     = 0.0;
  private double               dYMin     = 0.0;
  private plotMapping          plotMap   = new plotMapping();
  private plotWellStatusStruct stSymbols = new plotWellStatusStruct();

  private int                  iXGrid[]  = null;
  private int                  iYGrid[]  = null;

  private Image                imageLandGrid = null;
  private int                  IN        = 0;

  private int                  iAddBox   = 0;
  private int                  iAddStart = 0;
  private int                  iAddWidth = 0;

  private int                  iXStart   = 0;
  private int                  iXEnd     = 0;
  private int                  iYStart   = 0;
  private int                  iYEnd     = 0;

  /** Constructor geminiPlotWells()
   *  <p> This is the Constructor for this class.
   *  @param stStruct - Plot Data Structure.
   *  @param st       - Well Information Data List Structure
   *  @param stL      - Land Grid Data List Structure
   *  @param width    - Width of the Well Map
   *  @param height   - Height of the Well Map
   *  @param lbl      - Text Field to display Well Information.
   *  @param iGrid    - Plot Text with Grid or NOT.
   */

  public geminiPlotWells( Observable notifier,
                          cmnStruct stStruct,
                          iqstratHeadersListStruct st,
                          utilLandGridListStruct stL,
                          int width,
                          int height,
                          JLabel lbl,
                          int iGrid )
  {
    try
    {
	  this.notifier   = notifier;
      this.stStruct   = stStruct;
      this.lbl        = lbl;

      this.stPlot     = st;
      this.iRows      = stPlot.iCount;

      this.stLandGrid = stL;
      this.iGrid      = iGrid;

      this.iXBegin    = 0;
      this.iYBegin    = 0;
      this.iWidth     = width -iXBegin;
      this.iHeight    = height-iYBegin;

      jbInit();
    }
    catch (Exception e)
    {
      JOptionPane.showMessageDialog((Component) null,
                                    "geminiPlotWells(): " + e.getMessage(),
                                    "ERROR",
                                    JOptionPane.ERROR_MESSAGE);
    }
  }

  /** Method jbInit()
   *  <p> This method will create the Linear Plot.
   *  @throws Exception
   */

  public void jbInit() throws Exception
  {
    setSize(iWidth, iHeight);
    setBackground(new Color(250, 250, 250));

    iX      = new int[iRows];
    iY      = new int[iRows];
    iSymbol = new int[iRows];

    getMinMaxPosition();
    imageLandGrid = utilPlotLandGridImage.makePlotImage( stLandGrid,
                                                         iWidth,
                                                         iHeight,
                                                         dXMin, dXMax,
                                                         dYMin, dYMax,
                                                         iGrid );
    getData();
    getWellSymbol();

    addMouseListener(this);
    addMouseMotionListener(this);
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    notifier   = null; // Observable Object this Frame
    stStruct   = null;
    stPlot     = null;
    stLandGrid = null;
    lbl        = null;

    // Global Variables

    iX         = null;
    iY         = null;
    iSymbol    = null;

    // Well Select Number within well list

    iWells     = null;

    // -- Filter Wells By

    sLogType   = null;

    // Plot Variables

    if (plotMap != null)
      plotMap.close();
    plotMap = null;

    iXGrid        = null;
    iYGrid        = null;

    imageLandGrid = null;
  }

  /* ======================================================================== *
   * ------------------------- GET Methods ---------------------------------- *
   * ======================================================================== */

  /** Methods Get Minimum & Maximum Latitude & Longitude
   *  @dXMin = Minimum Longitude
   *  @dXMax = Maximum Longitude
   *  @dYMin = Minimum Latitude
   *  @dYMax = Maximum Latitude
   */

  public double getMinLatitude()  { return (dYMin); }
  public double getMaxLatitude()  { return (dYMax); }
  public double getMinLongitude() { return (dXMin); }
  public double getMaxLongitude() { return (dXMax); }

  /** Method getWell()
   *  <p> This method will return the selected well.
   */

  public iqstratHeadersStruct getWell()
  {
	iqstratHeadersStruct st = null;

	if (stPlot != null)
	{
	  if (iSelectedWell > -1)
      {
		st = stPlot.stItem[iSelectedWell];
      }
	}

	return (st);
  }

  /** Method getMinMaxPosition()
   *  <p> This method will determine the minimum and maximum position of the
   *      wells that have been passed in.
   */

  private void getMinMaxPosition()
  {
    int    i          = 0;
    int    j          = 0;
    int    iTemp      = 0;
    int    iFound     = 0;
    int    iStart     = 0;
    double dLongitude = 0.0;
    double dLatitude  = 0.0;
    double dMinLat    = 0.0;
    double dMaxLat    = 0.0;
    double dMinLong   = 0.0;
    double dMaxLong   = 0.0;

    double delX       = 0.0;
    double delY       = 0.0;

    double dTemp      = 0.0;
    double dLat       = 0.0;
    double dLong      = 0.0;

    for (i=0; i<stLandGrid.iCount; i++)
    {
      for (j=0; j<stLandGrid.stItem[i].iTotal; j++)
      {
        if (iStart == 0)
        {
          dMinLat  = stLandGrid.stItem[i].dLatitude[j];
          dMaxLat  = stLandGrid.stItem[i].dLatitude[j];
          dMinLong = stLandGrid.stItem[i].dLongitude[j];
          dMaxLong = stLandGrid.stItem[i].dLongitude[j];
          iStart   = 1;
        }

        if (dMinLat > stLandGrid.stItem[i].dLatitude[j])
          dMinLat  = stLandGrid.stItem[i].dLatitude[j];

        if (dMaxLat < stLandGrid.stItem[i].dLatitude[j])
          dMaxLat  = stLandGrid.stItem[i].dLatitude[j];

        if (dMinLong > stLandGrid.stItem[i].dLongitude[j])
          dMinLong  = stLandGrid.stItem[i].dLongitude[j];

        if (dMaxLong < stLandGrid.stItem[i].dLongitude[j])
          dMaxLong  = stLandGrid.stItem[i].dLongitude[j];
      }
    }

    for (i=0; i<iRows; i++)
    {
      dLatitude  = stPlot.stItem[i].dLatitude;
      dLongitude = stPlot.stItem[i].dLongitude;

      if ((Math.abs(dLongitude) > 0.0) &&
          (Math.abs(dLatitude)  > 0.0))
      {
        if (iFound == 0)
        {
          dXMin = dLongitude;
          dXMax = dXMin;

          dYMin = dLatitude;
          dYMax = dYMin;

          iFound = 1;
        }

        if (dLongitude < dXMin)
          dXMin = dLongitude;

        if (dLongitude > dXMax)
          dXMax = dLongitude;

        if (dLatitude < dYMin)
          dYMin = dLatitude;

        if (dLatitude > dYMax)
          dYMax =dLatitude;
      }
    }

    if (dXMin < dMinLong)
      dXMin = dMinLong;

    if (dXMax > dMaxLong)
      dXMax = dMaxLong;

    if (dYMin < dMinLat)
      dYMin = dMinLat;

    if (dXMax > dMinLat)
      dYMax = dMaxLat;

    if (IN == 0)
    {
      dXMin = dMinLong;
      dXMax = dMaxLong;
      dYMin = dMinLat;
      dYMax = dMaxLat;
    }

    double dX = 0.05 * (dXMax - dXMin);
    double dY = 0.05 * (dYMax - dYMin);

    dXMin = dXMin - dX;
    dXMax = dXMax + dX;
    dYMin = dYMin - dY;
    dYMax = dYMax + dY;

    delX  = Math.abs(dXMax - dXMin);
    delY  = Math.abs(dYMax - dYMin);

    dXMinOrig = dXMin;
    dXMaxOrig = dXMax;
    dYMinOrig = dYMin;
    dYMaxOrig = dYMax;

    dTemp = iWidth / (4*iHeight/5);
  }

  /** Method getData()
   *  <p> This method will convert the raw data to pixels so the data can be
   *      plotted.
   */

  public void getData()
  {
    int    iRatio;
    int    i,j,k;
    double dLongitude = 0.0;
    double dLatitude  = 0.0;
    int    iCont      = 0;


    for (i=0; i<iRows; i++)
    {
      dLongitude = stPlot.stItem[i].dLongitude;
      iRatio = (int)((double)iWidth * (dLongitude-dXMin) / (dXMax - dXMin));
      iX[i] = iXBegin + iRatio;
    }

    for (i=0; i<iRows; i++)
    {
      dLatitude = stPlot.stItem[i].dLatitude;
      iRatio = (int)((double)iHeight * (dLatitude - dYMin)/ (dYMax - dYMin));
      iY[i] = iYBegin + iHeight - iRatio;
    }
  }

  /** Method getWellSymbol()
   *  <p> This method will convert the Well Symbol to one of the recognized
   *      symbols.
   */

  private void getWellSymbol()
  {
    for (int i=0; i<iRows; i++)
    {
      iSymbol[i] = plot.plotWellStatusStruct.NONE;

      for (int j=0; j<stSymbols.iCount; j++)
      {
        if (stPlot.stItem[i].status.equals(stSymbols.status[j][0]))
          iSymbol[i] = stSymbols.iStatus[j];
      }
    }
  }

  /* ======================================================================== *
   * ------------------------- SET Methods ---------------------------------- *
   * ======================================================================== */

  /** Method clearWell()
   *  <p> This method will deselete well
   *  @param iPanel = the Panel cleared
   */

  public void clearWell( int iPanel )
  {
	iWells[iPanel] = -1;
	repaint();
  }

  /** Method setWell()
   *  <p> This method will set whether reference well is to be selected or
   *      the edit tops well is to be selected
   *  @param iWell = Well to be selected
   */

  public void setWell( int iWell ) { this.iWell = iWell; }

  /** Method setLogType()
   *  <p> This method will set the filter of the wells that are
   *      being plotted.
   *  @param filter = the filter the Well plot
   */

  public void setLogType(String filter)
  {
    sLogType = new String(filter);
    repaint();
  }

  /** Method setText()
   *  <p> This method will set the filter of the wells that are
   *      being plotted.
   *  @param filter = the filter to the Well plot
   */

  public void setText(int iAdd )
  {
	iShow = iAdd;
	repaint();
  }

  /** Method setWellList()
   *  <p> This method will refresh the list of wells to be plotted.
   *  @param st - Well Position List Data Structure.
   *  @param iGrid   = Plot Text with Grid or NOT.
   */


  public void setWellList(iqstratHeadersListStruct st, int iGrid)
  {
    stPlot  = st;
    iRows   = stPlot.iCount;

    iX      = new int[iRows];
    iY      = new int[iRows];
    iSymbol = new int[iRows];

    this.iGrid = iGrid;

    getMinMaxPosition();
    imageLandGrid = utilPlotLandGridImage.makePlotImage(stLandGrid,
                                                        iWidth,
                                                        iHeight,
                                                        dXMin, dXMax,
                                                        dYMin, dYMax,
                                                        iGrid);
    getData();
    getWellSymbol();

    repaint();
  }

  /** Method setField()
   *  <p> This method will set field KID to plot field back ground color
   *  @param sFieldKID = Field KID
   */

  public void setField(String sFieldKID)
  {
	this.sFieldKID = new String(sFieldKID);
    repaint();
  }

  /** Method compute()
   *  <p> This method will re-compute the Minimum & Maximum Latitude & Longitude
   */

  private void compute()
  {
    double dX = 0.0;
    double dY = 0.0;

    double dX0 = 0.0;
    double dX1 = 0.0;
    double dY0 = 0.0;
    double dY1 = 0.0;

//System.out.println("A "+dXMin+" "+dXMax+" - "+dYMin+" "+dYMax);
    dX0   = dXMin + (dXMax-dXMin) * (iXStart-iXBegin) / iWidth;
    dX1   = dXMin + (dXMax-dXMin) * (iXEnd-iXBegin) / iWidth;

    dY1   = dYMax - (dYMax-dYMin) * (iYStart-iYBegin) / iHeight;
    dY0   = dYMax - (dYMax-dYMin) * (iYEnd-iYBegin) / iHeight;
//System.out.println("B "+dX0+" "+dX1+" - "+dY0+" "+dY1);

    dXMin = dX0;
    dXMax = dX1;
    dYMin = dY0;
    dYMax = dY1;

    dX    = 0.05 * (dXMax - dXMin);
    dY    = 0.05 * (dYMax - dYMin);

    dXMin = dXMin - dX;
    dXMax = dXMax + dX;
    dYMin = dYMin - dY;
    dYMax = dYMax + dY;

    getData();
    imageLandGrid = utilPlotLandGridImage.makePlotImage( stLandGrid,
                                                         iWidth,
                                                         iHeight,
                                                         dXMin, dXMax,
                                                         dYMin, dYMax,
                                                         iGrid );

    repaint();
  }

  /** Method zoom_off()
   *  <p> This method will set the zoom off;
   */

  public void zoom_off() { iAction = ZOOM_OFF; }

  /** Method zoom_in()
   *  <p> This method will compute the Minimum & Maximum Latitude & Longitude
   */

  public void zoom_in() { iAction = ZOOM_IN; }

  /** Method zoom_out()
   *  <p> This method will reset the Minimum & Maximum Latitude & Longitude
   */

  public void zoom_out()
  {
    dXMin   = dXMinOrig;
    dXMax   = dXMaxOrig;
    dYMin   = dYMinOrig;
    dYMax   = dYMaxOrig;

    getData();
    imageLandGrid = utilPlotLandGridImage.makePlotImage( stLandGrid,
                                                         iWidth,
                                                         iHeight,
                                                         dXMin, dXMax,
                                                         dYMin, dYMax,
                                                         iGrid );

    iAction = ZOOM_OFF;

    repaint();
  }

  /* ======================================================================== *
   *  MouseListener Methods required when implementing the MouseListener Class.
   * ======================================================================== */

  /** Method mouseMoved()
   *  <p> This method will retrieve the location of the mouse pointer and
   *      convert the postion to latitude and longitude position. It will then
   *      display the text in the Volumetric Panel.
   * @param event - Mouse Moved Event
   */

  public void mouseMoved(MouseEvent event)
  {
    int    iXm       = event.getX();
    int    iYm       = event.getY();
    int    iTemp     = 0;
    int    iFound    = 0;
    double dTemp     = 0.0;
    double dX        = 0.0;
    double dY        = 0.0;
    String sPosition = new String("");
    String sLAS      = ""; // Well Contains LAS File Data          (L)
    String sTYPE     = ""; // Well Contains LAS File Data (L)

    if ((iXm > iXBegin) && (iXm < iXBegin + iWidth) &&
        (iYm > iYBegin) && (iYm < iYBegin + iHeight))
    {
      dX = dXMin + (dXMax - dXMin) * (double) (iXm - iXBegin)/(double) (iWidth);
      dTemp = dX * 1000.0;
      iTemp = (int) dTemp;
      dX = (double) iTemp / 1000.0;

      dY = dYMin - (dYMax - dYMin) * (double) (iYm - (iYBegin+iHeight)) /
          (double) (iHeight);
      dTemp = dY * 1000.0;
      iTemp = (int) dTemp;
      dY = (double) iTemp / 1000.0;

      iFound = 0;
      for (int i=0; i<iRows; i++)
      {
        sLAS = ""; // Well Contains LAS File Data (L)

        if ((iXm > (iX[i]-5)) && (iXm < (iX[i]+5)) &&
            (iYm > (iY[i]-5)) && (iYm < (iY[i]+5)))
        {
          if (stPlot.stItem[i].iLAS > 0)       { sLAS = new String("L"); }
          if (stPlot.stItem[i].ilitho_log > 0) { sLAS = new String("LITHO"); }
          if (stPlot.stItem[i].iGeo > 0)       { sLAS = new String("GEO"); }

          sPosition = new String(stPlot.stItem[i].sAPI   + "  " +
                                 stPlot.stItem[i].sName  +
                                 " Latitude: "  + stPlot.stItem[i].dLatitude +
                                 " Longitude: " + stPlot.stItem[i].dLongitude);

          if (sLogType.equals("ALL") || sLogType.equals(sLAS))
          {
            iFound = 1;
          }

          if (lbl != null)
            lbl.setForeground(Color.red);
        }
      }

      if (iFound == 0)
      {
        sPosition = new String("Latitude: " + dY + " Longitude: " + dX);
        if (lbl != null)
          lbl.setForeground(Color.black);
      }

      if (lbl != null)
        lbl.setText(sPosition);
    }
  }

  /** Method mouseDragged()
   *  <p> This method will dragg the mouse and draw the Rubber Band.
   * @param event - Mouse Moved Event
   */

  public void mouseDragged(MouseEvent event)
  {
    iAddBox = 1;
    showRubberBand(event);
  }

  public void mouseReleased(MouseEvent event)
  {
    iXEnd = event.getX();
    iYEnd = event.getY();

    showRubberBand(event);
    iAddBox = 0;

    if (iAction == ZOOM_IN)
      compute();
  }

  /** Method mousePressed(MouseEvent)
   *  <p> gets the current coordinates and creates a new rectangle,
   *      updates the rectangle to be drawn and
   *      calls repaint on the parent panel
   *  @param event - MouseEvent info
   */

  public void mousePressed(MouseEvent event)
  {
    iXStart = event.getX();
    iXEnd   = 0;
    iYStart = event.getY();
    iYEnd   = 0;
    iAddBox = 1;
    showRubberBand(event);
  }

  //public void mousePressed(MouseEvent event)  {}
  public void mouseEntered(MouseEvent event)  {}
  public void mouseExited(MouseEvent event)   {}

  /** Method mouseClicked(MouseEvent)
   *  <p> user clicked mouse get Well
   *  @param event - MouseEvent info
   */

  public void mouseClicked(MouseEvent event)
  {
    int    iXm    = event.getX();
    int    iYm    = event.getY();
    int    iFound = 0;
    String sURL   = "";
    String sLAS   = ""; // Well Contains LAS File Data (L)
    String sTYPE  = ""; // Well Contains LAS File Data (L)

    iXStart = event.getX();
    iYStart = event.getY();

    iFound = 0;
    for (int i=0; i<iRows; i++)
    {
      sLAS = ""; // Well Contains LAS File Data (L)

      if ((iXm > (iX[i]-5)) && (iXm < (iX[i]+5)) &&
          (iYm > (iY[i]-5)) && (iYm < (iY[i]+5)))
      {
        if ((sLogType.equals("L")) && (stPlot.stItem[i].iLAS > 0))
        {
		  sLAS = new String("L");
		}
        if ((sLogType.equals("LITHO")) && (stPlot.stItem[i].ilitho_log > 0))
        {
		  sLAS = new String("LITHO");
		}
        if ((sLogType.equals("GEO")) && (stPlot.stItem[i].iGeo > 0))
        {
		  sLAS = new String("GEO");
		}

        if ((iFound == 0) &&
            (sLogType.equals("ALL") || sLogType.equals(sLAS)))
        {
          iFound = 1;
        }

        if (iFound == 1)
        {
          if (stPlot.stItem[i] != null)
          {
			iSelectedWell = i;
			iWells[iWell] = i;
            repaint();

  	        if (notifier != null)
	          notifier.notifyObservers(new String("Well Selected"));
          }
        }
      }
    }
  }

  /** Method showRubberBand(MouseEvent)
   *  <p> This method is adapted from java.sun.com(MouseMotionEventDemo.java)
   *      updates the size of the current rectangle
   *  @param e - MouseEvent info
   */

  public void showRubberBand(MouseEvent e)
  {
    iXEnd = e.getX();
    iYEnd = e.getY();

    this.repaint();
  }

  /* ======================================================================== *
   *  Drawing Methods                                                         *
   * ======================================================================== */

  /** Method drawRubberBand()
   *  <p> This method will draw a box around the selected area.
   *  @param g - Graphics pointer.
   */

  public void drawRubberBand(Graphics g)
  {
    int iXStartMouse = iXStart;
    int iYStartMouse = iYStart;
    int iMouseWidth  = 0;
    int iMouseHeight = 0;

    if (iXEnd < iXStart)
    {
      iXStartMouse = iXEnd;
      iMouseWidth  = iXStart - iXEnd;
    }
    else
      iMouseWidth  = iXEnd   - iXStart;

    if (iYEnd < iYStart)
    {
      iYStartMouse = iYEnd;
      iMouseHeight = iYStart - iYEnd;
    }
    else
      iMouseHeight = iYEnd   - iYStart;

    if (iAddBox == 1)
    {
      g.setColor(Color.cyan);
      g.drawRect(iXStartMouse, iYStartMouse, iMouseWidth, iMouseHeight);

      g.setColor(Color.gray);
      g.drawRect(iXStartMouse+1, iYStartMouse+1, iMouseWidth, iMouseHeight);
    }
  }

  /** Method drawWells()
   *  <p> This method will draw the Linear Plot Curves.
   *  @param g - Graphics pointer.
   */

  public void drawWells(Graphics g)
  {
    int i, j;
    int    iSelect = 0;
    int    iLevel  = -1;
    String sLAS    = ""; // Well Contains LAS File Data (L)
    String sTYPE   = ""; // Well Contains LAS File Data (L)
    String sName   = ""; // Text to add to each well
    int    iCont   = 0;

    Font        fsb  = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm = g.getFontMetrics(fsb);

    g.setFont( fsb );
    g.setColor(Color.black);

    for (i=0; i<iRows; i++)
    {
	  iCont   = 0;
	  iSelect = 0;
	  for (j=0; j<_TOTAL_WELLS; j++)
	  {
	    if (i == iWells[j])
	    {
	      iSelect = 1;
		}
	  }

      sName = "";
      if ((iShow == _WELL_NAME) || (iShow == _API_NUMBER))
      {
		if (iShow == _WELL_NAME)
		  sName = stPlot.stItem[i].sName;
		else
		  sName = stPlot.stItem[i].sAPI;
	  }

      sLAS = ""; // Well Contains LAS File Data (L)

      if ((iX[i] >= iXBegin) && (iX[i] <= iXBegin+iWidth) &&
          (iY[i] >= iYBegin) && (iY[i] <= iYBegin+iHeight))
      {
//        iLevel = -1;
//        if (stPlot.stItem[i].iTops > 0)
//        {
//          iLevel = plot.plotMapping._POOR;
//	    }

        if ((sLogType.equals("L")) && (stPlot.stItem[i].iLAS > 0))
        {
		  sLAS  = new String("L");
		}

        if ((sLogType.equals("LITHO")) && (stPlot.stItem[i].ilitho_log > 0))
        {
		  sLAS = new String("LITHO");
		}

        if ((sLogType.equals("GEO")) && (stPlot.stItem[i].iGeo > 0))
        {
		  sLAS = new String("GEO");
		}

//        if (iCont == 1)
        {
		  g.setColor(Color.black);
		  g.drawString(sName, iX[i]+9, iY[i]);

          plotMap.drawWellSymbol(g, iSymbol[i], iX[i], iY[i]);
          plotMap.drawHalo(g, iLevel, iX[i], iY[i]);
          if (iSelect == 1)
            plotMap.drawHalo(g, plot.plotMapping._SELECTED, iX[i], iY[i]);
		}
      }
    }
  }

  /** Method drawField()
   *  <p> This method will draw the Field area.
   *  @param g - Graphics pointer.
   */

  public void drawField(Graphics g)
  {
	int   iCont   = 0;
	int   iRadius = 50;

    for (int i=0; i<iRows; i++)
    {
	  iCont   = 0;

      if ((iX[i] >= iXBegin) && (iX[i] <= iXBegin+iWidth) &&
          (iY[i] >= iYBegin) && (iY[i] <= iYBegin+iHeight))
      {
        if (stPlot.stItem[i].sField_kid.equals(sFieldKID))
        {
	      iCont = 1;
	    }

        if (iCont == 1)
        {
  		  g.setColor(new Color(1f, 0f, 0f, .1f));
	      g.fillOval(iX[i]-iRadius/2, iY[i]-iRadius/2, iRadius, iRadius);
		}
	  }
	}
  }

  /** Method drawProduction()
   *  <p> This method will draw the production area.
   *  @param g - Graphics pointer.
   *

  public void drawProduction(Graphics g)
  {
	int   iCont   = 0;
	int   iRadius = 50;
	float fRed    = 0;
	float fGreen  = 0;
	float fBlue   = 0;
	int   iRadMax = 50;
	double dMax   = 200;

    for (int i=0; i<iRows; i++)
    {
	  iCont   = 0;

      if ((iX[i] >= iXBegin) && (iX[i] <= iXBegin+iWidth) &&
          (iY[i] >= iYBegin) && (iY[i] <= iYBegin+iHeight))
      {
        if (stPlot.stItem[i].sOIL_kid.length() > 1)
        {
	      iCont = 1;
	    }

        if (iCont == 1)
        {
          if (iColor[i][0] > -1)
          {
			fRed   = (float) _COLORS[iColor[i][0]][0] / (float) 255;
	 		fGreen = (float) _COLORS[iColor[i][0]][1] / (float) 255;
			fBlue  = (float) _COLORS[iColor[i][0]][2] / (float) 255;
			g.setColor(new Color(fRed, fGreen, fBlue, .1f));

			for (int m=0; m<stList[i].iCount; m++)
			{
			  if ((stList[i].stItem[m].iYear == iYear) && (stList[i].stItem[m].sProduct.equals("OIL")))
			  {
			    iRadius = (int) ( iRadMax * stList[i].stItem[m].data / dMax);
			    if (iRadius > 50)
			      iRadius = 50;
	            g.fillOval(iX[i]-iRadius/2, iY[i]-iRadius/2, iRadius, iRadius);
			  }
			}
		  }
        }
	  }
	}
  }
*/
  /** Method paint()
   *  <p> This method will create the Linear Plot.
   *  @param g - Graphics pointer.
   */

  public void paint(Graphics g)
  {
	int   iYears  = 1;
	int   iYear   = 0;

    g.drawImage(imageLandGrid, 0, 0, iWidth, iHeight, null);
    drawField(g);
    drawWells(g);
    if (iAction == ZOOM_IN)
      drawRubberBand(this.getGraphics());
  }
}
