/*
 * @plotWellSymbolFrame.java Version 1.1 12/12/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package plot;

import java.awt.*;
import javax.swing.*;

import plot.plotMapping;

/** Class plotWellSymbolFrame
 *  <p> This class create the Plot Well Symbol Image Frame
 *
 *  @version 1.1 12/12/2012
 *  @author  John R. Victorine
 */

public class plotWellSymbolFrame extends JFrame
{
  private String                   sMessage = new String("");
  private int                      iWidth;
  private int                      iHeight;

  /** Constructor plotWellSymbolFrame()
   *  <p> This is the Constructor for this Class.
   *  @param iWidth - Width of the Well Map
   *  @param iHeight - Height of the Well Map
   */

  public plotWellSymbolFrame(int iWidth,  int iHeight)
  {
    try
    {
      jbInit(iWidth, iHeight);
    }
    catch(Exception e)
    {
      sMessage = new String("plotWellSymbolFrame(): " + e.getMessage());
      JOptionPane.showMessageDialog((Component) null,
                                    sMessage,
                                    "ERROR",
                                    JOptionPane.ERROR_MESSAGE);
    }
  }

  /** METHOD jbInit()
   *  <p> This Method creates the Well Symbol Description Plot Frame
   *  @param iWidth - Width of the Well Map
   *  @param iHeight - Height of the Well Map
   */

  private void jbInit(int iWidth,  int iHeight) throws Exception
  {
    JScrollPane  jScrollPane = new JScrollPane();

    this.setTitle("Well Plot Symbols");
    this.getContentPane().setLayout(new BorderLayout());

    plotWellSymbolFrameDisplayImage pSymbols =
        new plotWellSymbolFrameDisplayImage( iWidth, iHeight );

    jScrollPane.getViewport().add(pSymbols, null);

    this.getContentPane().add(jScrollPane, BorderLayout.CENTER);

    /*
     * Set the Dimension and Location of this Frame.
     */

    this.setSize(new Dimension(330, 510)); //350));

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    this.setLocation((d.width - this.getSize().width) / 2,
                     (d.height - this.getSize().height) / 2);

    this.setResizable(false);
    this.setVisible(true);
  }
}

/** CLASS plotWellSymbolFrameDisplayImage
 *  <p> This Class will display the Well Symbol Description Plot Image.
 */

class plotWellSymbolFrameDisplayImage extends Canvas
{
  int   iX;
  int   iY;
  plotMapping plotWells = null;

  /** CONSTRUCTOR wellPlotFrameDisplayImage()
   *  <p> This constructor for this class.
   *  @param i  - Image to be displayed.
   *  @param ix - Width of Image
   *  @param iy - Height of Image
   */

  public plotWellSymbolFrameDisplayImage(int ix, int iy)
  {
    iX = ix;
    iY = iy;
    this.setSize(iX, iY);

    plotWells = new plotMapping();
  }

  /** METHOD paint()
   *  <p> This method will paint the image on the Image Panel.
   *  @param g - Graphics Pointer
   */

  public void paint(Graphics g)
  {
    g.setColor(Color.white);
    g.fillRect(0, 0, iX, iY);

    plotWells.drawSymbolDescription(g);
  }
}

/*
 *  @version 1.1 12/12/2012
 *  @author  John Victorine
 */
