/*
 * @productionControlFrame.java Version 1.1 09/12/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package production.gui;

import java.net.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.Observer;
import java.util.Observable;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import gui.guiSearchDirectoryFrame;           // File Search Dialog

import production.productionStruct;           // Production Data Structure
import production.productionListStruct;       // Production List Structure
import production.productionBuildPlotData;    // Production Build Plot Data
import production.declineStruct;              // Decline Curve Data Structure
import production.gui.productionDeclinePanel; // Decline Curve Calculator Panel

import production.plot.productionPlotStruct;  // Production Plot Data Structure
import production.gui.productionPlotFrame;    // Production Plot Dialog

/** Class productionControlFrame
 *  <p> This Class will create the Production Control Plot Frame.
 *
 *  @version 1.1 09/12/2013
 *  @author  John R. Victorine
 */

public class productionControlFrame extends JFrame implements ActionListener, Observer
{
  public static final  int LEASE    = iqstrat.iqstratInfoStruct.LEASE;
  public static final  int FIELD    = iqstrat.iqstratInfoStruct.FIELD;
  public static final  int COUNTY   = iqstrat.iqstratInfoStruct.COUNTY;
  public static final  int OPERATOR = iqstrat.iqstratInfoStruct.OPERATOR;

  public static final  int OIL      = production.productionStruct.OIL;
  public static final  int GAS      = production.productionStruct.GAS;

  public static final  int OFF      = cmn.cmnStruct.OFF;
  public static final  int ON       = cmn.cmnStruct.ON;

  private static final int XMIN     = 0;
  private static final int XMAX     = 1;
  private static final int XINC     = 2;

  private static final int YMIN     = 3;
  private static final int YMAX     = 4;
  private static final int YINC     = 5;

  private static final int DAY      = production.math.declineMath.DAILY;
  private static final int MONTH    = production.math.declineMath.MONTHLY;
  private static final int YEAR     = production.math.declineMath.YEARLY;

  // Input Variables

  private productionListStruct    stList    = null; // Production List Structure

  // Global Variables

  private Observable              notifier  = null; // Observalbles Object

  private productionBuildPlotData pData     = null; // Build Plot Data
  private productionDeclinePanel  pDecline  = null; // Decline Curve Calculator Panel

  private declineStruct           stDecline = null; // Production Plot Data Structure
  private productionPlotStruct    stPlot    = null; // Production Plot Data Structure
  private productionPlotFrame     plot      = null; // Production Plot Dialog

  private guiSearchDirectoryFrame pSearch   = null; // Search for Directory to Save PNG

  private int iType    = LEASE;  // Type of Production Plot
  private int iProduct = OIL;    // Product to display

  private boolean iAvailable[]  = { false, false };

  private String sTitle1        = "";   // First line of the plot title
  private String sTitle2        = "";   // Second line of the plot title
  private String sTitle3        = "";   // Third line of the plot title

  private double dXmax          = 0.0;  // Maximum X-Axis
  private double dXmin          = 0.0;  // Minimum X-Axis
  private double dXincr         = 0.0;  // Increment along X-Axis

  private double dYmax          = 0.0;  // Maximum Y-Axis
  private double dYmin          = 0.0;  // Minimum Y-Axis
  private int    iYCycles       = 1;    // Number of Log Cycles (Only of Log Scales)

  // Screen Width

  private int    iControlWidth  = 425;  // Control Panel Width
  private int    iControlHeight = 825;  // Control Panel Height
  private int    iScreenWidth   = 0;    // Screen Width
  private int    iScreenHigh    = 0;    // Scree Height

  // Save Path & Filename

  private static final int _PDF = 0;
  private static final int _XML = 1;
  private static final int _CSV = 2;

  private int    iSave = -1;

  private String sPath = new String("");
  private String sName = new String("");

  // Plot Screen Location & Size

  private int    iFrameX        = -1;   // Plot Frame X-Coordintate
  private int    iFrameY        = -1;   // Plot Frame Y-Coordintate
  private int    iFrameWidth    = -1;   // Plot Frame Width
  private int    iFrameHeight   = -1;   // Plot Frame Height

  // Global Frame Widgets

  // -- Top Panel

  // .. Titles

  private JTextField txtTitle1 = new JTextField();  // 1st Title
  private JTextField txtTitle2 = new JTextField();  // 2nd Title
  private JTextField txtTitle3 = new JTextField();  // 3rd Title

  // .. X-Axis Text fields

  private JLabel     lblXAxis  = new JLabel();      // Column Title
  private JLabel     lblXmin   = new JLabel();      // Minimum X Label
  private JLabel     lblXmax   = new JLabel();      // Maximum X Label
  private JLabel     lblXincr  = new JLabel();      // Increment X Label

  private JTextField txtXmin   = new JTextField();  // Minimum X-Axis
  private JTextField txtXmax   = new JTextField();  // Maximum X-Axis
  private JTextField txtXincr  = new JTextField();  // Increment X-Axis

  // .. Y-Axis Text fields

  private JLabel     lblYAxis  = new JLabel();      // Column Title
  private JLabel     lblYmin   = new JLabel();      // Minimum Y Label
  private JLabel     lblYmax   = new JLabel();      // Maximum Y Label
  private JLabel     lblYincr  = new JLabel();      // Increment Y Label

  private JTextField txtYmin   = new JTextField();  // Minimum Y-Axis
  private JTextField txtYmax   = new JTextField();  // Maximum Y-Axis
  private JTextField txtYincr  = new JTextField();  // Increment Y-Axis

  // -- Menus

  // .. File Button Menu

  private static final int    PRODUCTION = production.plot.productionPlot.PRODUCTION;
  private static final int    CUMULATIVE = production.plot.productionPlot.CUMULATIVE;
  private static final int    BOTH       = production.plot.productionPlot.BOTH;

  private static final String srb[] = {
    "Production Only",
    "Cumulative Only",
    "Production & Cumulative" };

  private JRadioButtonMenuItem rb[] = null;         // Production ON/OFF

  private static final int    _RAW     = 0;
  private static final int    _MONTHLY = 1;
  private static final int    _DAILY   = 2;
  private static final int    _WELLS   = 3;
  private static final int    _DECLINE = 4;

  private static final String srbOther[]  = { "OFF", "ON" };
  private static final String slblOther[] = {
    "Raw Production",
    "Monthly Average Production",
    "Daily Average Production",
    "Number of Wells",
    "Decline Curve" };

  private JRadioButtonMenuItem rbOther[][] = null;  // Other Curves ON/OFF

  private static final String smProduct[]  = { "OIL", "GAS" };

  private JRadioButtonMenuItem mProduct[]  = null;  // Product OIL or GAS

  private JMenuItem mXML     = null;  // Save as XML File
  private JMenuItem mCSV     = null;  // Save as CSV File
  private JMenuItem mPDF     = null;  // Create a PDF of Profile Plot
  private JMenuItem mExit    = null;  // Exit Profile Plot

  /** Constructor productionControlFrame()
   *  <p> This is the Constructor for this class.
   *  @param stList = Production Data List Structure
   */

  public productionControlFrame( productionListStruct stList )
  {
    try
    {
	  this.stList = stList;

      if (stList != null)
      {
        for (int i=0; i<stList.iCount; i++)
        {
		  if (stList.stItem[i].sProduct.equals(
			    production.productionStruct.PRODUCT[OIL]))
		  {
			iAvailable[OIL] = true;
		  }

		  if (stList.stItem[i].sProduct.equals(
			    production.productionStruct.PRODUCT[GAS]))
		  {
			iAvailable[GAS] = true;
		  }
		}

        for (int i=0; i<4; i++)
        {
    	  if ((stList.sType).toUpperCase().equals(production.productionStruct.DATA_BY[i]))
		  {
		    iType = i;
		  }
        }

        pData = new productionBuildPlotData(iType);
        pData.setProductionData(stList);
	  }

      jbInit();

   	  stDecline = pDecline.getData();

      addWindowListener( new productionControlFrame_WindowListener() );

      plot();
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	int i, j;

    JPanel panel             = new JPanel();
    JScrollPane scroll       = new JScrollPane();
    JPanel pnlTop            = new JPanel();

    ButtonGroup group        = new ButtonGroup();
    ButtonGroup groupOther[] = null;
    ButtonGroup groupProduct = new ButtonGroup();

    // Build Menus

    JMenuBar menuBar     = new JMenuBar();
    JMenu    menuFile    = new JMenu("File");
    JMenu    menuProduct = new JMenu("Product");
    JMenu    menuCurves  = new JMenu("Plot Curves ON/OFF");

    // -- File Menu Items

    mXML = new JMenuItem("Save as XML File");
    mXML.addActionListener(this);

    mCSV = new JMenuItem("Save as CSV File");
    mCSV.addActionListener(this);

    mPDF = new JMenuItem("Create PNG Image File");
    mPDF.addActionListener(this);

    mExit = new JMenuItem("Exit");
    mExit.addActionListener(this);

    // -- Add Menu Items to File Menu

    menuFile.add( mXML );
    menuFile.add( mCSV );
    menuFile.addSeparator();
    menuFile.add( mPDF );
    menuFile.addSeparator();
    menuFile.add( mExit );

    // -- Product

    mProduct = new JRadioButtonMenuItem[2];

    for (i=0; i<2; i++)
    {
	  mProduct[i] = new JRadioButtonMenuItem();
      mProduct[i].setFont(new java.awt.Font("Dialog", 1, 12));
      mProduct[i].setText(smProduct[i]);
      mProduct[i].addActionListener(this);

      groupProduct.add(mProduct[i]);

      if (iAvailable[i])
      {
        menuProduct.add(mProduct[i]);
	  }
    }

    if (iAvailable[OIL])
    {
	  mProduct[OIL].setSelected(true);
	  iProduct = OIL;
	}
	else if (iAvailable[GAS])
	{
	  mProduct[GAS].setSelected(true);
	  iProduct = GAS;
	}

    getPlotData();

    // -- Curve ON/OFF Menu Items

    rb = new JRadioButtonMenuItem[3];

    for (i=0; i<3; i++)
    {
	  rb[i] = new JRadioButtonMenuItem();
      rb[i].setFont(new java.awt.Font("Dialog", 1, 12));
      rb[i].setText(srb[i]);
      rb[i].addActionListener(this);
      if (i==2) rb[i].setSelected(true);

      group.add(rb[i]);

      if (i!=1)
        menuCurves.add(rb[i]);
    }

    menuCurves.addSeparator();

    rbOther    = new JRadioButtonMenuItem[5][2];
    groupOther = new ButtonGroup[5];

    for (i=0; i<5; i++)
    {
      groupOther[i] = new ButtonGroup();

	  for (j=0; j<2; j++)
	  {
  	    rbOther[i][j] = new JRadioButtonMenuItem();
        rbOther[i][j].setFont(new java.awt.Font("Dialog", 1, 12));
        rbOther[i][j].setText(srbOther[j] + ": " + slblOther[i]);
        rbOther[i][j].addActionListener(this);
        if ((i!=4) && (j==0)) { rbOther[i][j].setSelected(true); }
        if ((i==4) && (j==1)) { rbOther[i][j].setSelected(true); }

        groupOther[i].add(rbOther[i][j]);

        menuCurves.add(rbOther[i][j]);
	  }

      menuCurves.addSeparator();
	}

    // -- Add Menus to Menu Bar

    menuBar.add(menuFile);
    if ((iAvailable[OIL]) || (iAvailable[GAS]))
      menuBar.add(menuProduct);
    menuBar.add(menuCurves);
    this.setJMenuBar(menuBar);

    // Create an Observable

    notifier = new productionControlFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Production Plot Frame

    this.setTitle("Producton Plot Control");
    this.getContentPane().setLayout(new BorderLayout());

    panel.setLayout( new BorderLayout() );
    panel.setPreferredSize(new Dimension(iControlWidth-50, iControlHeight-75)); //50));
    panel.setBackground(Color.white);

    // -- Panel Top

    pnlTop = buildPlotControlPanel();

    // Decline Curve Analysis Panel

    pDecline = new productionDeclinePanel(notifier, stPlot);

    // Add Widgets to Frame

    this.getContentPane().add( scroll,    BorderLayout.CENTER );
    scroll.getViewport().add( panel,      null);

    panel.add(pnlTop,     BorderLayout.NORTH);
    panel.add(pDecline,   BorderLayout.CENTER);

    // Display the Frame

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    int iScreenHigh  = d.height-50;

    this.setSize(new Dimension(iControlWidth, iControlHeight)); //iScreenHigh));
    this.setLocation(1,1);

//    this.setResizable(false);
    this.setVisible(true);
  }

  /** Method buildPlotControlPanel()
   *  <p> This method will build the Plot Control Panel
   */

  private JPanel buildPlotControlPanel()
  {
	int i, j;

	JPanel panel             = new JPanel();

    JPanel pnlTitles         = new JPanel();
    JPanel pnlTitle1         = new JPanel();
    JPanel pnlTitle2         = new JPanel();
    JPanel pnlTitle3         = new JPanel();

    JPanel pnlAxes           = new JPanel();
    JPanel pnlXAxis          = new JPanel();
    JPanel pnlXAxisCenter    = new JPanel();
    JPanel pnlXMinimum       = new JPanel();
    JPanel pnlXMaximum       = new JPanel();
    JPanel pnlXIncrement     = new JPanel();

    JPanel pnlYAxis          = new JPanel();
    JPanel pnlYAxisCenter    = new JPanel();
    JPanel pnlYMinimum       = new JPanel();
    JPanel pnlYMaximum       = new JPanel();
    JPanel pnlYIncrement     = new JPanel();

    TitledBorder titledBorderTitle1 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "1st Plot Title");
    titledBorderTitle1.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderTitle2 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "2nd Plot Title");
    titledBorderTitle2.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderTitle3 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "3rd Plot Title");
    titledBorderTitle3.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderXAxis = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "X-Axis");
    titledBorderXAxis.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderYAxis = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Y-Axis");
    titledBorderYAxis.setTitleFont(new java.awt.Font("Dialog", 1, 13));

	// Build Production Plot Control Panel

    panel.setLayout(new BorderLayout());

    // .. Titles Panel

    pnlTitles.setLayout(new GridLayout(3,1));

    // .. .. 1st Title Panel

    pnlTitle1.setLayout(new BorderLayout());
    pnlTitle1.setBorder(titledBorderTitle1);

    txtTitle1.setText(sTitle1);
    txtTitle1.setFont(new java.awt.Font("Dialog", 1, 12));
    txtTitle1.addFocusListener(new productionControlFrameFocusAdapter(this));

    pnlTitle2.setLayout(new BorderLayout());
    pnlTitle2.setBorder(titledBorderTitle2);

    txtTitle2.setText(sTitle2);
    txtTitle2.setFont(new java.awt.Font("Dialog", 1, 12));
    txtTitle2.addFocusListener(new productionControlFrameFocusAdapter(this));


    pnlTitle3.setLayout(new BorderLayout());
    pnlTitle3.setBorder(titledBorderTitle3);

    txtTitle3.setText(sTitle3);
    txtTitle3.setFont(new java.awt.Font("Dialog", 1, 12));
    txtTitle3.addFocusListener(new productionControlFrameFocusAdapter(this));

    // -- Axes Panel

    pnlAxes.setLayout(new GridLayout());

    // .. X-Axis Panel

    pnlXAxis.setLayout(new BorderLayout());
    pnlXAxis.setBorder(titledBorderXAxis);

    // .. .. X-Axis Label Panel

    lblXAxis.setText("Year");
    lblXAxis.setHorizontalAlignment(SwingConstants.CENTER);
    lblXAxis.setFont(new java.awt.Font("Dialog", 1, 13));

    // .. .. X-Axis Center Panel

    pnlXAxisCenter.setLayout(new GridLayout(3,1));

    // .. - .. X-Axis Minimum Panel

    pnlXMinimum.setLayout(new BorderLayout());

    lblXmin.setText("Minimum:  ");
    lblXmin.setFont(new java.awt.Font("Monospaced", 1, 12));

    txtXmin.setText("" + (int) dXmin);
    txtXmin.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXmin.setFont(new java.awt.Font("Dialog", 1, 12));
    txtXmin.addFocusListener(new productionControlFrameFocusAdapter(this));

    // .. - .. X-Axis Maximum Panel

    pnlXMaximum.setLayout(new BorderLayout());

    lblXmax.setText("Maximum:  ");
    lblXmax.setFont(new java.awt.Font("Monospaced", 1, 12));

    txtXmax.setText("" + (int) dXmax);
    txtXmax.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXmax.setFont(new java.awt.Font("Dialog", 1, 12));
    txtXmax.addFocusListener(new productionControlFrameFocusAdapter(this));

    // .. - .. X-Axis Increment Panel

    pnlXIncrement.setLayout(new BorderLayout());

    lblXincr.setText("Increment:");
    lblXincr.setFont(new java.awt.Font("Monospaced", 1, 12));

    txtXincr.setText("" + (int) dXincr);
    txtXincr.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXincr.setFont(new java.awt.Font("Dialog", 1, 12));
    txtXincr.addFocusListener(new productionControlFrameFocusAdapter(this));

    // .. Y Axis Panel

    pnlYAxis.setLayout(new BorderLayout());
    pnlYAxis.setBorder(titledBorderYAxis);

    // .. .. Y-Axis Label Panel

    lblYAxis.setText("Production/Cumulative");
    lblYAxis.setHorizontalAlignment(SwingConstants.CENTER);
    lblYAxis.setFont(new java.awt.Font("Dialog", 1, 13));


    // .. .. Y-Axis Center Panel

    pnlYAxisCenter.setLayout(new GridLayout(3,1));

    // .. - .. Y-Axis Minimum Panel

    pnlYMinimum.setLayout(new BorderLayout());

    lblYmin.setText("Minimum:  ");
    lblYmin.setFont(new java.awt.Font("Monospaced", 1, 12));

    txtYmin.setText("" + (int) dYmin);
    txtYmin.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYmin.setFont(new java.awt.Font("Dialog", 1, 12));
    txtYmin.addFocusListener(new productionControlFrameFocusAdapter(this));

    // .. - .. Y-Axis Maximum Panel

    pnlYMaximum.setLayout(new BorderLayout());

    lblYmax.setText("Maximum:  ");
    lblYmax.setFont(new java.awt.Font("Monospaced", 1, 12));

    txtYmax.setText("" + (int) dYmax);
//    txtYmax.setEditable(false);
    txtYmax.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYmax.setFont(new java.awt.Font("Dialog", 1, 12));
    txtYmax.addFocusListener(new productionControlFrameFocusAdapter(this));

    // .. - .. Y-Axis Increment Panel

    pnlYIncrement.setLayout(new BorderLayout());

    lblYincr.setText("Cycles:   ");
    lblYincr.setFont(new java.awt.Font("Monospaced", 1, 12));

    txtYincr.setText("" + iYCycles);
    txtYincr.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYincr.setFont(new java.awt.Font("Dialog", 1, 12));
    txtYincr.addFocusListener(new productionControlFrameFocusAdapter(this));

    // Add Widgets to Frame

    // -- Titles

    panel.add(pnlTitles,                  BorderLayout.NORTH);
    pnlTitles.add(pnlTitle1,              null);
    pnlTitle1.add(txtTitle1,              BorderLayout.CENTER);

    pnlTitles.add(pnlTitle2,              null);
    pnlTitle2.add(txtTitle2,              BorderLayout.CENTER);

    pnlTitles.add(pnlTitle3,              null);
    pnlTitle3.add(txtTitle3,              BorderLayout.CENTER);

    // -- Axes

    panel.add(pnlAxes,                    BorderLayout.CENTER);

    // .. X-Axis

    pnlAxes.add(pnlXAxis,                 null);
    pnlXAxis.add(lblXAxis,                BorderLayout.NORTH);

    pnlXAxis.add(pnlXAxisCenter,          BorderLayout.CENTER);
    pnlXAxisCenter.add(pnlXMinimum,       null);
    pnlXMinimum.add(lblXmin,              BorderLayout.WEST);
    pnlXMinimum.add(txtXmin,              BorderLayout.CENTER);

    pnlXAxisCenter.add(pnlXMaximum,       null);
    pnlXMaximum.add(lblXmax,              BorderLayout.WEST);
    pnlXMaximum.add(txtXmax,              BorderLayout.CENTER);

    pnlXAxisCenter.add(pnlXIncrement,     null);
    pnlXIncrement.add(lblXincr,           BorderLayout.WEST);
    pnlXIncrement.add(txtXincr,           BorderLayout.CENTER);

    // .. Y-Axis

    pnlAxes.add(pnlYAxis,                 null);
    pnlYAxis.add(lblYAxis,                BorderLayout.NORTH);

    pnlYAxis.add(pnlYAxisCenter,          BorderLayout.CENTER);
    pnlYAxisCenter.add(pnlYMinimum,       null);
    pnlYMinimum.add(lblYmin,              BorderLayout.WEST);
    pnlYMinimum.add(txtYmin,              BorderLayout.CENTER);

    pnlYAxisCenter.add(pnlYMaximum,       null);
    pnlYMaximum.add(lblYmax,              BorderLayout.WEST);
    pnlYMaximum.add(txtYmax,              BorderLayout.CENTER);

    pnlYAxisCenter.add(pnlYIncrement,     null);
    pnlYIncrement.add(lblYincr,           BorderLayout.WEST);
    pnlYIncrement.add(txtYincr,           BorderLayout.CENTER);

	return (panel);
  }

  /** METHOD productionControlFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class productionControlFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      close();
    }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

//    if (stList != null)
//      stList.delete();
    stList     = null;

    // Global Variables

    notifier   = null;

    if (pData != null)
      pData.close();
    pData      = null; // Build Plot Data

    if (pDecline != null)
      pDecline.close();
    pDecline   = null; // Decline Curve Calculator Panel

    if (stPlot != null)
      stPlot.delete();
    stPlot     = null; // Production Plot Data Structure

    if (plot != null)
      plot.close();
    plot = null;

    if (pSearch != null)
      pSearch.close();
    pSearch = null;

    iAvailable = null;

    sTitle1    = null; // First line of the plot title
    sTitle2    = null; // Second line of the plot title
    sTitle3    = null; // Third line of the plot title

    // Save Path & Filename

    sPath      = null;
    sName      = null;

    // Global Frame Widgets

    // -- Top Panel

    // .. Titles

    txtTitle1 = null; // 1st Title
    txtTitle2 = null; // 2nd Title
    txtTitle3 = null; // 3rd Title

    // .. X-Axis Text fields

    lblXAxis  = null; // Column Title
    lblXmin   = null; // Minimum X Label
    lblXmax   = null; // Maximum X Label
    lblXincr  = null; // Increment X Label

    txtXmin   = null; // Minimum X-Axis
    txtXmax   = null; // Maximum X-Axis
    txtXincr  = null; // Increment X-Axis

    // .. Y-Axis Text fields

    lblYAxis  = null; // Column Title
    lblYmin   = null; // Minimum Y Label
    lblYmax   = null; // Maximum Y Label
    lblYincr  = null; // Increment Y Label

    txtYmin   = null; // Minimum Y-Axis
    txtYmax   = null; // Maximum Y-Axis
    txtYincr  = null; // Increment Y-Axis

    // -- Menus

    // .. File Button Menu

    rb        = null; // Production ON/OFF
    rbOther   = null; // Other Curves ON/OFF

    mProduct  = null; // Product OIL or GAS

    mXML      = null; // Save as XML File
    mCSV      = null; // Save as CSV File
    mPDF      = null; // PDF of Profile Plot
    mExit     = null; // Exit

    dispose();
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  private void getPlotData()
  {
	if (stPlot != null)
	  stPlot.delete();
	stPlot = null;

    stPlot = pData.buildProductionPlotData(production.productionStruct.PRODUCT[iProduct]);
    stPlot.iProduct = iProduct;

    if (stPlot != null)
    {
      sTitle1  = stPlot.sTitle1;  // First line of the plot title
      sTitle2  = stPlot.sTitle2;  // Second line of the plot title
      sTitle3  = stPlot.sTitle3;  // Third line of the plot title

      dXmax    = stPlot.dXmax;     // Maximum X-Axis
      dXmin    = stPlot.dXmin;     // Minimum X-Axis
      dXincr   = stPlot.dXincr;    // Increment along X-Axis

      dYmax    = stPlot.dYmax;     // Maximum Y-Axis
      dYmin    = stPlot.dYmin;     // Minimum Y-Axis
      iYCycles = stPlot.iYCycles;  // Number of Log Cycles (Only of Log Scales)
	}
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setData()
   * <p> This method will set the extra data structures
   */

  public void setData( productionListStruct stList) { this.stList = stList; }

  /** Method setProduct()
   *  <p> This method will reset load a different product
   *  @param iProduct this will set the product data
   */

  private void setProduct( int iProduct )
  {
	this.iProduct = iProduct;

    rb[BOTH].setSelected(true);

	getPlotData();
	setScreen();

    pDecline.clear();
	pDecline.setData( stPlot );
	pDecline.resetScreen();
	stDecline = pDecline.getData();

    plot.setData(stPlot, stDecline);
  }

  /** Method setScreen()
   *  <p> This method will reload the Production Plot Control widgets
   */

  private void setScreen()
  {
    txtTitle1.setText( sTitle1 );
    txtTitle2.setText( sTitle2 );
    txtTitle3.setText( sTitle3 );

    txtXmin.setText(  "" + (int) dXmin );
    txtXmax.setText(  "" + (int) dXmax );
    txtXincr.setText( "" + (int) dXincr );

    txtYmin.setText(  "" + dYmin );
    txtYmax.setText(  "" + dYmax );
    txtYincr.setText( "" + iYCycles );
  }

  /** Method setTitle()
   *  <p> This method will set the Title Variables
   *  @param iTitle = the Title Number 1,2,3
   *  @param sTitle = the Title
   */

  private void setTitle(int iTitle, String sTitle)
  {
	switch (iTitle)
	{
	  case 1:
	    sTitle1 = new String( sTitle );
	    if (stPlot != null) { stPlot.sTitle1 = new String( sTitle ); }
	    break;
	  case 2:
	    sTitle2 = new String( sTitle );
	    if (stPlot != null) { stPlot.sTitle2 = new String( sTitle ); }
	    break;
	  case 3:
	    sTitle3 = new String( sTitle );
	    if (stPlot != null) { stPlot.sTitle3 = new String( sTitle ); }
	    break;
	}
  }

  /* =============================================================== *
   * ------------------ SET CURVES ON/OFF METHODS ------------------ *
   * =============================================================== */

  /** Method setMainCurve(int iCurve)
   *  <p> This method will set the Curves ON/OFF depending on selection
   */

  private void setMainCurve(int iCurve)
  {
	if (plot != null) { plot.setProductionCurve(iCurve); }
  }

  /** Method setCurve(int iCurve)
   *  <p> This method will set the Curves ON/OFF depending on selection
   */

  private void setCurve(int iCurve)
  {
	int iValue = -1;

	if (rbOther[iCurve][0].isSelected())
	  iValue = OFF;

	if (rbOther[iCurve][1].isSelected())
	  iValue = ON;

	if (iValue > -1)
	{
      if (plot != null)
      {
	    switch (iCurve)
	    {
          case _RAW:
            plot.setRawDataCurve(iValue);
            break;
          case _MONTHLY:
            plot.setMonthlyAvgCurve(iValue);
            break;
          case _DAILY:
            plot.setDailyAvgCurve(iValue);
            break;
          case _WELLS:
           	plot.setWellCurve(iValue);
            break;
          case _DECLINE:
            plot.setDeclineCurve(iValue);
            break;
		}
	  }
	}
  }

  /* =============================================================== *
   * ----------------- SET CURVESX XY AXES METHODS ----------------- *
   * =============================================================== */

  /** Method setX()
   *  <p> This method will set the X-Axis Values
   *  @param dXmax    = The Maximum X Values
   *  @param dXmin    = The Minimum X Values
   *  @param dXincr   = The Increment X Value
   *  @param iXCycles = the Number of X-Cycles
   */

  public void setX(double dXmax, double dXmin, double dXincr, int iXCycles)
  {
	if (plot != null) { plot.setX( dXmax, dXmin, dXincr, iXCycles); }
  }

  /** Method setY()
   *  <p> This method will set the Y-Axis Values
   *  @param dYmax    = The Maximum Y Values
   *  @param dYmin    = The Minimum Y Values
   *  @param dYincr   = The Increment Y Value
   *  @param iYCycles = the Number of Y-Cycles
   */

  public void setY(double dYmax, double dYmin, double dYincr, int iYCycles)
  {
	if (plot != null) { plot.setY( dYmax, dYmin, dYincr, iYCycles); }
  }

  /* ===================================================================== *
   * ------------------------------ plot --------------------------------- *
   * ===================================================================== */

  /** Method plot()
   * <p> This method will create the stratigraphic plot
   */

  private void plot()
  {
    if (plot != null)
    {
      iFrameX      = plot.getLocation().x;
      iFrameY      = plot.getLocation().y;
      iFrameWidth  = plot.getSize().width;
      iFrameHeight = plot.getSize().height;

      plot.close();
      plot = null;
	}

    if (stPlot != null)
    {
      plot = new productionPlotFrame( this,
                                      iControlWidth,
                                      iFrameX,
                                      iFrameY,
                                      iFrameWidth,
                                      iFrameHeight,
                                      stPlot,
                                      stDecline );
    }
  }

  /* ===================================================================== *
   * -------------------------- CREATE FILES ----------------------------- *
   * ===================================================================== */

  /** Method createXML()
   *  <p> This method will create a ASCII XML File of Production Data
   */

  private void createXML()
  {
	String sXML = "";

	sXML = production.io.WriteProductionXMLFile.buildXML( stList );
	production.io.WriteProductionXMLFile.write( sPath, sName+".xml", sXML );
  }

  /** Method createCSV()
   *  <p> This method will create a ASCII CSV File of Production Data
   */

  private void createCSV()
  {
	String sCSV = "";

	sCSV = production.io.WriteProductionXMLFile.buildCSV( stList );
	production.io.WriteProductionXMLFile.write( sPath, sName+".csv", sCSV );
  }

  /** Method createPDF()
   *  <p> This method will create a PDF Image of Production Plot
   */

  private void createPDF()
  {
    production.io.productionIO_Files.createImage( sPath, sName, plot.getImage() );
    production.io.productionIO_Files.createHTML( sPath, sName );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
	int i=0;
	int j=0;

    for (i=0; i<3; i++)
    {
      if (event.getSource() == rb[i]) { setMainCurve(i); }
	}

    for (i=0; i<5; i++)
    {
	  for (j=0; j<2; j++)
	  {
		if (event.getSource() == rbOther[i][j]) { setCurve(i); }
	  }
	}

    if (event.getSource() == mProduct[OIL]) { setProduct( OIL ); }  // OIL
    if (event.getSource() == mProduct[GAS]) { setProduct( GAS ); }  // GAS

    if (event.getSource() == mXML) // Save as XML File
    {
	  iSave = _XML;
	  if (sPath.length() == 0)
        sPath = production.io.productionIO_Files.getHomeDir();

      if (sName.length() == 0)
        sName = new String( "production" );

      pSearch = new guiSearchDirectoryFrame(notifier, sPath, sName, ".xml");
	}

    if (event.getSource() == mCSV) // Save as CSV File
    {
	  iSave = _CSV;
	  if (sPath.length() == 0)
        sPath = production.io.productionIO_Files.getHomeDir();

      if (sName.length() == 0)
        sName = new String( "production" );

      pSearch = new guiSearchDirectoryFrame(notifier, sPath, sName, ".csv");
	}

    if (event.getSource() == mPDF) // Create a PDF of Profile Plot
    {
	  iSave = _PDF;
	  if (sPath.length() == 0)
        sPath = production.io.productionIO_Files.getHomeDir();

      if (sName.length() == 0)
        sName = new String( "production" );

      pSearch = new guiSearchDirectoryFrame(notifier, sPath, sName, ".png");
    }

    // Exit

    if (event.getSource() == mExit)  { close(); }
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
	int    iField  = -1;
	String sText   = "";
	double dText   = 0.0;
	int    iTemp   = 0;
	double dTemp   = 0.0;
	int    iCycle  = 0;
    int    iCycleL = 0;
    int    iCycleU = 0;
    double diff    = 0.0;
    int    iError  = 0;
    String sError  = "";

    if (e.getSource() == txtTitle1) { setTitle(1, txtTitle1.getText() ); }
    if (e.getSource() == txtTitle2) { setTitle(2, txtTitle2.getText() ); }
    if (e.getSource() == txtTitle3) { setTitle(3, txtTitle3.getText() ); }

    if (e.getSource() == txtXmin)
    {
	  iField = XMIN;
	  sText  = txtXmin.getText();
	}

    if (e.getSource() == txtXmax)
    {
	  iField = XMAX;
	  sText  = txtXmax.getText();
	}

    if (e.getSource() == txtXincr)
    {
	  iField = XINC;
	  sText  = txtXincr.getText();
	}

    if (e.getSource() == txtYmin)
    {
	  iField = YMIN;
	  sText  = txtYmin.getText();
	}

    if (e.getSource() == txtYmax)
    {
	  iField = YMAX;
	  sText  = txtYmax.getText();
	}

    if (e.getSource() == txtYincr)
    {
	  iField = YINC;
	  sText  = txtYincr.getText();
	}

	if (iField > -1)
	{
	  if (cmn.cmnString.isNumeric(sText))
	  {
		dText = cmn.cmnString.stringToDouble(sText);
//System.out.println("X: "+dXmin+" "+dXmax+" "+dXincr+" Y: "+dYmin+" "+dYmax+" "+iYCycles);

		switch (iField)
		{
          case XMIN:
            if (dText <= dXmax)
              dXmin  = dText;
            else
            {
			  iError = -1;
			  sError = "Minimum Value must be less than the Maximum";
              txtXmin.setText( "" + (int) dXmin);  // Minimum X-Axis
			}
            break;
          case XMAX:
            if (dText >= dXmin)
              dXmax = dText;
            else
            {
			  iError = -1;
			  sError = "Minimum Value must be less than the Maximum";
              txtXmax.setText( "" + (int) dXmax);  // Maximum X-Axis
		    }
            break;
          case XINC:
            dTemp = (dXmax-dXmin)/dText;
            if (dTemp < 20.0)
              dXincr = dText;
            else
            {
			  iError = -1;
			  sError = "Increment value too small for Range of Time";
		      txtXincr.setText("" + (int) dXincr); // Increment X-Axis
		    }
            break;
          case YMIN:
            if ((dText <= dYmax) && (dText > 0.0))
              dYmin = dText;
            else if (dText <= 0.0)
            {
			  iError = -1;
			  sError = "The Minimum value can not be 0.0 or less than 0.0.";
              txtXmin.setText( "" + (int) dXmin);  // Minimum X-Axis
			}
            else
            {
			  iError = -1;
			  sError = "Minimum Value must be less than the Maximum";
              txtXmin.setText( "" + (int) dXmin);  // Minimum X-Axis
			}
            break;
          case YMAX:
            if (dText >= dYmin)
              dYmax = dText;
            else
            {
			  iError = -1;
			  sError = "Minimum Value must be less than the Maximum";
              txtXmax.setText( "" + (int) dXmax);  // Maximum X-Axis
		    }
            break;
          case YINC:
            if (dText < 7.0)
              dXincr = dText;
            else
            {
			  iError = -1;
			  sError = "Too many Cycles for this data";
		      txtXincr.setText("" + (int) dXincr); // Increment X-Axis
		    }
            break;
		}

        if (iError == -1)
        {
          JOptionPane.showMessageDialog((Component) null,
      	                                sError,
		                                "ERROR",
		                                JOptionPane.ERROR_MESSAGE );
		}
		else
		{
		  switch (iField)
		  {
            case XMIN:
              dXmin = (double) ((int) dText);
              txtXmin.setText( "" + (int) dXmin); // Minimum X-Axis

              iTemp  = (int) (1.0 + (dXmax-dXmin)/20.0);
              dXincr = (double) iTemp;

		      txtXincr.setText("" + (int) iTemp); // Increment X-Axis

		      plot.setX( dXmax, dXmin, dXincr, 1);
              break;
            case XMAX:
              dXmax = (double) ((int) dText);
              txtXmax.setText( "" + (int) dXmax);  // Maximum X-Axis

              iTemp  = (int) (1.0 + (dXmax-dXmin)/20.0);
              dXincr = (double) iTemp;

		      txtXincr.setText("" + (int) iTemp); // Increment X-Axis

		      plot.setX( dXmax, dXmin, dXincr, 1);
              break;
            case XINC:
              dText = (double) ((int) dText);
              dTemp = (dXmax-dXmin)/dText;

              if (dTemp < 20.0)
                dXincr = dText;

		      txtXincr.setText("" + (int) dXincr); // Increment X-Axis

		      plot.setX( dXmax, dXmin, dXincr, 1);
              break;

            case YMIN:
              dYmin    = dText;
   	          dYmax    = cmn.cmnString.stringToDouble(txtYmax.getText());

    	      iCycleL  = (int) Math.floor( Math.log(dYmin) / Math.log(10.0));
  	          iCycleU  = (int) Math.ceil( Math.log(dYmax) / Math.log(10.0));

              dYmin    = Math.pow(10.0, iCycleL);
              dYmax    = Math.pow(10.0, iCycleU);
  	          iYCycles = iCycleU-iCycleL;

              txtYmin.setText(""+ dYmin);
              txtYmax.setText(""+ dYmax);
              txtYincr.setText("" + (int) iYCycles); // Increment X-Axis

		      plot.setY( dYmax, dYmin, 1.0, iYCycles );
              break;
            case YMAX:
              dYmax    = dText;
   	          dYmin    = cmn.cmnString.stringToDouble(txtYmin.getText());

    	      iCycleL  = (int) Math.floor( Math.log(dYmin) / Math.log(10.0));
  	          iCycleU  = (int) Math.ceil( Math.log(dYmax) / Math.log(10.0));

              dYmin    = Math.pow(10.0, iCycleL);
              dYmax    = Math.pow(10.0, iCycleU);
  	          iYCycles = iCycleU-iCycleL;

              txtYmin.setText(""+ dYmin);
              txtYmax.setText(""+ dYmax);
              txtYincr.setText("" + (int) iYCycles); // Increment X-Axis

		      plot.setY( dYmax, dYmin, 1.0, iYCycles );
              break;
            case YINC:
              iYCycles = (int) dText;

   	          dYmin    = cmn.cmnString.stringToDouble(txtYmin.getText());
    	      iCycleL  = (int) Math.floor( Math.log(dYmin) / Math.log(10.0));
              iCycleU  = iCycleL + iYCycles;

              dYmax    = Math.pow(10.0, iCycleU);
              txtYmax.setText(""+ dYmax);
              txtYincr.setText("" + (int) iYCycles); // Increment X-Axis

		      plot.setY( dYmax, dYmin, 1.0, iYCycles );
              break;
		  }
	    }
	  }
	  else
	  {
		sError = new String( sText + " is an invalid numeric entry" );
		JOptionPane.showMessageDialog( (Component) null,
		                               sError,
		                               "ERROR",
		                               JOptionPane.ERROR_MESSAGE );

		switch (iField)
		{
	      case XMIN:
	        txtXmin.setText("" + (int) dXmin);
	        break;

	      case XMAX:
	        txtXmax.setText("" + (int) dXmax);
	        break;

	      case XINC:
	        txtXincr.setText("" + (int) dXincr);
	        break;

	      case YMIN:
	        txtYmin.setText("" + (int) dYmin);
	        break;

	      case YMAX:
	        txtYmax.setText("" + (int) dYmax);
	        break;

	      case YINC:
	        txtXincr.setText("" + (int) iYCycles);
	        break;
	    }
	  }
	}
  }

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg      = new String((String) arg);
    int    iRate     = YEAR;

    if (sArg.equals("Decline Curve Data Computed"))
    {
	  if (pDecline != null)
	  {
		stDecline = pDecline.getData();
		iRate     = pDecline.getRate();

		switch (iRate)
		{
          case DAY:
            rbOther[_DAILY][1].setSelected(true);
            setCurve(_DAILY);
            break;
          case MONTH:
            rbOther[_MONTHLY][1].setSelected(true);
            setCurve(_MONTHLY);
            break;
          case YEAR:
            break;
		}

		plot.setDeclineData( stDecline );
		plot.refreshDecline();
	  }
	}

    if (sArg.equals("Extend Decline Curve"))
    {
      dXmax  = 1 + pDecline.getDeclineYear();
      dXincr = 1 + (dXmax-dXmin) / 21.0;

      txtXmax.setText( "" + (int) dXmax);  // Maximum X-Axis
      txtXincr.setText("" + (int) dXincr); // Increment X-Axis

      dXmin  = cmn.cmnString.stringToDouble( txtXmin.getText() );
      dXmax  = cmn.cmnString.stringToDouble( txtXmax.getText() );
      dXincr = cmn.cmnString.stringToDouble( txtXincr.getText() );

      setX( dXmax, dXmin, dXincr, 1);
    }

    if (sArg.equals("Directory Path Selected"))
    {
      if (pSearch != null)
      {
        sPath = new String( pSearch.getPath() );
        sName = new String( pSearch.getFile() );
      }

      switch (iSave)
      {
        case _PDF:
          createPDF();
          break;
        case _XML:
          createXML();
          break;
        case _CSV:
          createCSV();
          break;
	  }

      iSave = -1;

      if (pSearch != null)
        pSearch.close();
      pSearch = null;
    }
  }
}

/** CLASS productionControlFrameFocusAdapter()
  *  <p> This Class will handle Actions when focus is lost on a textfield
  */

class productionControlFrameFocusAdapter extends java.awt.event.FocusAdapter
{
  productionControlFrame adaptee;

  productionControlFrameFocusAdapter(productionControlFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}


/** CLASS productionControlFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class productionControlFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 09/12/2013
 *  @author  John Victorine
 */
