/*
 * @productionMapCSVFrame.java Version 1.1 09/09/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package production.gui;

import java.io.*;
import java.lang.*;
import java.util.*;
import java.util.Observable;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import production.io.productionCSVFile;
import production.productionListStruct;

/** Class productionMapCSVFrame
 *  <p> This Class will allow the user to load data files from
 *      their PC or the KGS Database
 *
 *  @version 1.1 09/09/2013
 *  @author  John R. Victorine
 */

public class productionMapCSVFrame extends JFrame implements ActionListener
{
  public static final int LEASE    = iqstrat.iqstratInfoStruct.LEASE;
  public static final int FIELD    = iqstrat.iqstratInfoStruct.FIELD;
  public static final int COUNTY   = iqstrat.iqstratInfoStruct.COUNTY;
  public static final int OPERATOR = iqstrat.iqstratInfoStruct.OPERATOR;

  // Input Variables

  private Observable notifier    = null;
  private String     sReadMethod = null;
  private String     sDirectory  = null;
  private String     sFilename   = null;

  // Global Frame Variables

  public int     iType  = LEASE;

  private String sLine1 = "";
  private String sLine2 = "";

  private int iRow = 3;
  private int iColumn[] = { 0, 0, 0, 0, 0, 0 };

  private productionCSVFile    pFILE  = new productionCSVFile();
  private productionListStruct stList = null;

  // Global Frame Widgets

  private JRadioButton rbLease    = new JRadioButton(); // Lease
  private JRadioButton rbField    = new JRadioButton(); // Field
  private JRadioButton rbCounty   = new JRadioButton(); // County
  private JRadioButton rbOperator = new JRadioButton(); // Operator

  private JLabel     lblName      = new JLabel();       // Label of Name
  private JTextField txtName      = new JTextField();   // Name of Search

  private JLabel     lblDOR       = new JLabel();       // Label of DOR
  private JTextField txtDOR       = new JTextField();   // Lease DOR

  private JButton btnLoad   = new JButton();
  private JButton btnCancel = new JButton();
  private JButton btnHelp   = new JButton();

  private JTextField txtStart = new JTextField();

  private JPanel       pnl[]    = null;
  private JPanel       pnl1a[]  = null;
//  private JPanel       pnl2a[]  = null;
  private JLabel       lbl[]    = null;
  private JTextField   txt[]    = null;

  /** Constructor productionMapCSVFrame()
   *  <p> This is the Constructor for this class.
   *  @param notifier    = Observable
   *  @param sReadMethod = File Read Method
   *  @param sDirectory  = The Directory Path to the File
   *  @param sFilename   = The Filename
   */

  public productionMapCSVFrame( Observable notifier,
                                String     sReadMethod,
                                String     sDirectory,
                                String     sFilename )
  {
    try
    {
      this.notifier    = notifier;
      this.sReadMethod = sReadMethod;
      this.sDirectory  = sDirectory;
      this.sFilename   = sFilename;

      jbInit();

      if (sLine1 != null) { mapColumns( sLine1 ); }

      if (sLine2 != null)
      {
		mapColumns( sLine2 );
	  }
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	String str        = "";
    JPanel pnlTop     = new JPanel();
    JPanel pnlButtons = new JPanel();
    JPanel pnlCenter  = new JPanel();

    JPanel pnlInfo    = new JPanel();
    JPanel pnlType    = new JPanel();
    JPanel pnlName    = new JPanel();
    JPanel pnlHeader  = new JPanel();

    JPanel pnlLines   = new JPanel();
    JPanel pnlLine1   = new JPanel();
    JPanel pnlLine2   = new JPanel();
    JPanel pnlCHeader = new JPanel();
    JPanel pnlColumns = new JPanel();
    JPanel pnlStart   = new JPanel();
    JPanel pnl1       = new JPanel();
    JPanel pnlMap     = new JPanel();

    JLabel lblStart         = new JLabel();
    JLabel lblNote          = new JLabel();
    JLabel lblRockDataColumn = new JLabel();
    JLabel lblFileColumn    = new JLabel();

    JTextArea txtLine1 = new JTextArea();
    JTextArea txtLine2 = new JTextArea();

    JScrollPane scrollLine1   = new JScrollPane();
    JScrollPane scrollLine2   = new JScrollPane();
    JScrollPane scrollColumns = new JScrollPane();

    ButtonGroup group         = new ButtonGroup();

    TitledBorder titledBorder1st = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "1st Line of Comma Delimited File:");

    TitledBorder titledBorder2nd = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "2nd Line of Comma Delimited File:");

    TitledBorder titledBorderRock = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Production Data Columns:");

    titledBorder1st.setTitleFont(new java.awt.Font("Dialog", 1, 11));
    titledBorder2nd.setTitleFont(new java.awt.Font("Dialog", 1, 11));
    titledBorderRock.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    this.setTitle("Map File Column Number to Production Data Column");
    this.getContentPane().setLayout(new BorderLayout());

    pnlButtons.setBorder(BorderFactory.createEtchedBorder());

    btnLoad.setFont(new java.awt.Font("Dialog", 1, 11));
    btnLoad.setPreferredSize(new Dimension(90, 25));
    btnLoad.setText("Load Data");
    btnLoad.addActionListener(this);

    btnCancel.setFont(new java.awt.Font("Dialog", 1, 11));
    btnCancel.setPreferredSize(new Dimension(90, 25));
    btnCancel.setText("Cancel");
    btnCancel.addActionListener(this);

    btnHelp.setFont(new java.awt.Font("Dialog", 1, 11));
    btnHelp.setPreferredSize(new Dimension(90, 25));
    btnHelp.setText("Help");
    btnHelp.addActionListener(this);

    pnlTop.setLayout(new BorderLayout());

    pnlInfo.setLayout(new BorderLayout());

    // .. .. Type Panel

    pnlType.setLayout( new GridLayout() );

    rbLease.setFont(new java.awt.Font("Dialog", 1, 11));
    rbLease.setText( "Lease" );
    rbLease.setSelected(true);
    rbLease.addActionListener(this);

    rbField.setFont(new java.awt.Font("Dialog", 1, 11));
    rbField.setText( "Field" );
    rbField.addActionListener(this);

    rbCounty.setFont(new java.awt.Font("Dialog", 1, 11));
    rbCounty.setText( "County" );
    rbCounty.addActionListener(this);

    rbOperator.setFont(new java.awt.Font("Dialog", 1, 11));
    rbOperator.setText( "Operator" );
    rbOperator.addActionListener(this);

    // .. .. Name Panel

    pnlHeader.setLayout( new BorderLayout() );

    pnlName.setLayout( new BorderLayout() );

    lblName.setText("Name: ");
    lblName.setFont(new java.awt.Font("Dialog", 1, 12));

    txtName.setText("");
    txtName.setFont(new java.awt.Font("Dialog", 1, 12));

    pnlLines.setLayout(new GridLayout(2,1));

    pnlLine1.setBorder(titledBorder1st);
    pnlLine1.setLayout(new BorderLayout());

    txtLine1.setEditable(false);
    txtLine1.setLineWrap(true);
    txtLine1.setText("");
    txtLine1.setRows(3);
    txtLine1.setWrapStyleWord(true);

    pnlLine2.setBorder(titledBorder2nd);
    pnlLine2.setLayout(new BorderLayout());

    txtLine2.setEditable(false);
    txtLine2.setLineWrap(true);
    txtLine2.setText("");
    txtLine2.setRows(3);
    txtLine2.setWrapStyleWord(true);

    pnlCenter.setLayout(new BorderLayout());
    pnlCenter.setBorder(titledBorderRock);

    pnlCHeader.setLayout(new BorderLayout());

    lblStart.setFont(new java.awt.Font("Dialog", 1, 11));
    lblStart.setText("Start Reading Data at Row");

    txtStart.setText("3");
    txtStart.setColumns(4);
    txtStart.setHorizontalAlignment(SwingConstants.TRAILING);
    txtStart.addFocusListener(new productionMapCSVFrameFocusAdapter(this));

    lblNote.setFont(new java.awt.Font("Dialog", 1, 11));
    lblNote.setText("Assume Row & Column Count is 1,2,3 ...");

    pnlColumns.setLayout(new BorderLayout());

    // Build Map Panel

    pnlMap.setLayout(new GridLayout(production.productionTools._TOTAL+1,1));

    // .. Add Header to Map Panel

    pnl1.setLayout(new BorderLayout());

    lblRockDataColumn.setFont(new java.awt.Font("Dialog", 1, 11));
    lblRockDataColumn.setText("Production Data Column Name");

    lblFileColumn.setFont(new java.awt.Font("Dialog", 1, 11));
    lblFileColumn.setText("File Column Number");

    pnlMap.add(pnl1, null);
    pnl1.add(lblRockDataColumn, BorderLayout.WEST);
    pnl1.add(lblFileColumn, BorderLayout.EAST);

    // .. Add the Regions Columns to Map Panel

    pnl   = new JPanel[production.productionTools._TOTAL];
    pnl1a = new JPanel[production.productionTools._TOTAL];
//    pnl2a = new JPanel[production.productionTools.TOTAL];
    lbl   = new JLabel[production.productionTools._TOTAL];
    txt   = new JTextField[production.productionTools._TOTAL];

    for (int i=0; i<production.productionTools._TOTAL; i++)
    {
      pnl[i] = new JPanel();
      pnl[i].setLayout(new BorderLayout());
      pnl[i].setBorder(BorderFactory.createEtchedBorder());

      pnl1a[i] = new JPanel();
      pnl1a[i].setLayout(new BorderLayout());

//      pnl2a[i] = new JPanel();
//      pnl2a[i].setLayout(new GridLayout());

      str = new String( production.productionTools.CURVES[i][2] );
 /*
      if ((i >= production.productionTools._LI) &&
          (i <= production.productionTools._HSO4))
      {
        str = new String( str + " ( " +
                          production.productionTools.CURVES[i][1] +
                          " )" );
	  }
*/
      lbl[i] = new JLabel();
      lbl[i].setFont(new java.awt.Font("Dialog", 1, 11));
      lbl[i].setText("  " + str);

      txt[i] = new JTextField();
      txt[i].setText("0");
      txt[i].setColumns(4);
      txt[i].setHorizontalAlignment(SwingConstants.TRAILING);
      txt[i].addFocusListener(new productionMapCSVFrameFocusAdapter(this));

      pnlMap.add(pnl[i],     null);
      pnl[i].add(pnl1a[i],   BorderLayout.CENTER);
      pnl1a[i].add(lbl[i],   BorderLayout.CENTER);
      pnl[i].add(txt[i],     BorderLayout.EAST);
    }

    // Add 1st & 2nd Line to Frame

    pFILE.Open(util.utilFileIO.DIRECTORY, sDirectory, sFilename);
    pFILE.get2Rows();
    pFILE.Close();

    sLine1 = new String( pFILE.getLine1() );
    sLine2 = new String( pFILE.getLine2() );

    txtLine1.setText( sLine1 );
    txtLine2.setText( sLine2 );

    // Attach Widgets to Frame

    this.getContentPane().add(pnlTop,     BorderLayout.NORTH);
    pnlTop.add(pnlLines,                  BorderLayout.NORTH);
    pnlLines.add(pnlLine1,                null);
    pnlLine1.add(scrollLine1,             BorderLayout.CENTER);
    scrollLine1.getViewport().add(txtLine1, null);

    pnlLines.add(pnlLine2,                null);
    pnlLine2.add(scrollLine2,             BorderLayout.CENTER);
    scrollLine2.getViewport().add(txtLine2, null);

    pnlTop.add(pnlInfo,                   BorderLayout.SOUTH);
    pnlInfo.add(pnlType,                  BorderLayout.NORTH);
    pnlType.add( rbLease,                 null );
    pnlType.add( rbField,                 null );
    pnlType.add( rbCounty,                null );
    pnlType.add( rbOperator,              null );

    group.add( rbLease );
    group.add( rbField );
    group.add( rbCounty );
    group.add( rbOperator );

    pnlInfo.add(pnlHeader,                BorderLayout.SOUTH);
    pnlHeader.add( pnlName,               BorderLayout.CENTER );
    pnlName.add( lblName,                 BorderLayout.WEST );
    pnlName.add( txtName,                 BorderLayout.CENTER );

    this.getContentPane().add(pnlButtons, BorderLayout.SOUTH);
    pnlButtons.add(btnLoad,               null);
    pnlButtons.add(btnCancel,             null);
//    pnlButtons.add(btnHelp,   null);

    this.getContentPane().add(pnlCenter,  BorderLayout.CENTER);
    pnlCenter.add(pnlCHeader,             BorderLayout.NORTH);
    pnlCHeader.add(pnlStart,              BorderLayout.WEST);
    pnlStart.add(lblStart,                null);
    pnlStart.add(txtStart,                null);
    pnlCHeader.add(lblNote,               BorderLayout.EAST);

    pnlCenter.add(pnlColumns,             BorderLayout.CENTER);
    pnlColumns.add(scrollColumns,         BorderLayout.CENTER);
    scrollColumns.getViewport().add(pnlMap, null);

    // Display the Frame

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    this.setSize(new Dimension(400, 500));
    this.setLocation( 10, 10 );
//    this.setLocation((d.width  - this.getSize().width) / 2,
//                     (d.height - this.getSize().height) / 2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /* ====================================================================== *
   * ----------------------------- GET METHODS ---------------------------- *
   * ====================================================================== */

  /** Method getData()
   * <p> This method will return tops list
   * @return stList = production data list structure
   */

  public productionListStruct getData() { return (stList); }

  /** Method mapColumns()
   *  <p> This method will try to parse and map the columns of the text
   *      file to the production columns.
   *  @param sData = Line
   */

  private void mapColumns( String sData )
  {
	int i, j, k;
    int    iCount   = 0;
    int    iFound   = 0;
    int    iNext    = 0;
    int    iColumns = 0;
    String sTemp    = "";
    String sCol     = "";
    StringTokenizer stToken = null;
    String sTokens[] = null;

    sTemp    = new String(sData.replace('\t', ' '));
    sTokens  = production.io.productionCSVFile.splitDataLine( sTemp, "," );

    iColumns = 0;
    if (sTokens != null)
      iColumns = sTokens.length;

    for (i=0; i<iColumns; i++)
    {
      sTemp = sTokens[i];
      sTemp = new String(sTemp.toUpperCase());
      sTemp = new String(sTemp.trim());

      for  (j=0; j<production.productionTools._TOTAL; j++)
      {
		for (k=1; k<3; k++)
		{
          sCol = new String( production.productionTools.CURVES[j][k] );
          sCol = new String( sCol.toUpperCase() );

		  if (sTemp.equals(sCol))
		  {
            iNext      = i+1;
			txt[j].setText(""+iNext);
		  }
		}
      }
    }

    for (i=0; i<production.productionTools._TOTAL; i++)
    {
      iColumn[i] = cmn.cmnString.stringToInt(txt[i].getText());
    }
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** Method load()
   * <p> This method will load the read and parse the regions file
   */

  private void load()
  {
    pFILE.Open(util.utilFileIO.DIRECTORY, sDirectory, sFilename);
    pFILE.getData(iRow, iColumn);
    pFILE.Close();

    stList = pFILE.getData();
    stList.sName = new String( txtName.getText() );
    stList.sType = new String( production.productionStruct.DATA_BY[iType] );

    notifier.notifyObservers(new String("Production CSV File Loaded"));
  }

  /** Method close()
   * <p> This method will set global variables to null in order to force JAVA
   *     to free unwanted memory.
   */

  public void close()
  {
    notifier    = null;
    sReadMethod = null;
    sDirectory  = null;
    sFilename   = null;

    sLine1      = null;
    sLine2      = null;

    iColumn     = null;

    if (pFILE != null)
      pFILE.delete();
    pFILE = null;

    stList = null;

    btnLoad     = null;
    btnCancel   = null;
    btnHelp     = null;
    txtStart    = null;

    pnl      = null;
    pnl1a    = null;
//    pnl2a    = null;
    lbl      = null;
    txt      = null;

    dispose();
  }

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == rbLease)       { iType = LEASE; }    // By Lease
    if (event.getSource() == rbField)       { iType = FIELD; }    // By Field
    if (event.getSource() == rbCounty)      { iType = COUNTY; }   // By County
    if (event.getSource() == rbOperator)    { iType = OPERATOR; } // By Operator

    if (event.getSource() == btnLoad) { load(); }
    if (event.getSource() == btnHelp)
    {
//      util.BrowserControl.displayURL( cmn.cmnStruct.PC_CORE );
    }

    if (event.getSource() == btnCancel) { close(); }
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
    int    i        = 0;
    int    iNumeric = 0;
    String sMessage = new String("");
    String sTemp    = new String("");

    if (e.getSource() == txtStart)
    {
      iNumeric = 1;
      sTemp    = txtStart.getText();
      sMessage = new String("Starting Row Number is a Numeric Field");
    }

    for (i=0; i<production.productionTools._TOTAL; i++)
    {
      if (e.getSource() == txt[i])
      {
        iNumeric = 1;
        sTemp    = txt[i].getText();
        sMessage = new String("Column Number is a Numeric Field");
      }
    }

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
        JOptionPane.showMessageDialog((Component) null,
                                       sMessage,
                                       "ERROR",
                                       JOptionPane.ERROR_MESSAGE);
      }
      else
      {
        if (e.getSource() == txtStart)
        {
          iRow = cmn.cmnString.stringToInt(txtStart.getText());
        }

        for (i=0; i<production.productionTools._TOTAL; i++)
        {
          if (e.getSource() == txt[i])
          {
            iColumn[i] = cmn.cmnString.stringToInt(txt[i].getText());
          }
        }
      }
    }
  }
}

/** CLASS productionMapCSVFrameFocusAdapter()
  *  <p> This Method will handle Actions when focus is lost on a textfield
  */

class productionMapCSVFrameFocusAdapter extends java.awt.event.FocusAdapter
{
  productionMapCSVFrame adaptee;

  productionMapCSVFrameFocusAdapter(productionMapCSVFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/*
 *  @version 1.1 09/09/2013
 *  @author  John Victorine
 */
