/*
 * @productionMath.java Version 1.1 09/13/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package production.math;

import java.math.BigDecimal;

/** Class productionMath
 *  <p> This Class allows the ability to remove round off errors
 *      in the basic math package
 *
 *  @version 1.0 09/13/2013
 *  @author  John R. Victorine
 */

public class productionMath
{
  public static final int CEILING = java.math.BigDecimal.ROUND_CEILING;
  public static final int FLOOR   = java.math.BigDecimal.ROUND_FLOOR;

  /** Method add()
   *  This method will add two numbers and return the sum
   *
   *  dSum = dA + dB
   *
   *  @param  dA   = the variable A
   *  @param  dB   = the variable B
   *  @return dVal = the Sum of A & B
   */

  public static double add( double dA, double dB )
  {
    double     dVal  = 0.0;
    BigDecimal bgSum = new BigDecimal( "0.0" );
    BigDecimal bgA   = new BigDecimal( new String(""+dA) );
    BigDecimal bgB   = new BigDecimal( new String(""+dB) );

    bgSum   = bgA.add(bgB);
    dVal    = bgSum.doubleValue();

    return (dVal);
  }

  /** Method subtract()
   *  This method will add two numbers and return the sum
   *
   *  dVal = dA - dB
   *
   *  @param  dA   = the variable A
   *  @param  dB   = the variable B
   *  @return dVal = A minus B
   */

  public static double subtract( double dA, double dB )
  {
    double     dVal  = 0.0;
    BigDecimal bgSum = new BigDecimal( "0.0" );
    BigDecimal bgA   = new BigDecimal( new String(""+dA) );
    BigDecimal bgB   = new BigDecimal( new String(""+dB) );

    bgSum   = bgA.subtract(bgB);
    dVal    = bgSum.doubleValue();

    return (dVal);
  }

  /** Method multiply()
   *  This method will add two numbers and return the sum
   *
   *  dVal = dA * dB
   *
   *  @param  dA   = the variable A
   *  @param  dB   = the variable B
   *  @return dVal = the A multiplied by B
   */

  public static double multiply( double dA, double dB )
  {
    double     dVal  = 0.0;
    BigDecimal bgSum = new BigDecimal( "0.0" );
    BigDecimal bgA   = new BigDecimal( new String(""+dA) );
    BigDecimal bgB   = new BigDecimal( new String(""+dB) );

    bgSum   = bgA.multiply(bgB);
    dVal    = bgSum.doubleValue();

    return (dVal);
  }

  /** Method divide()
   *  This method will add two numbers and return the sum
   *
   *  dVal = dA / dB
   *
   *  @param  dA   = the variable A
   *  @param  dB   = the variable B
   *  @return dVal = the A divided by B
   */

  public static double divide( double dA, double dB )
  {
    double     dVal  = 0.0;
    BigDecimal bgSum = new BigDecimal( "0.0" );
    BigDecimal bgA   = new BigDecimal( new String(""+dA) );
    BigDecimal bgB   = new BigDecimal( new String(""+dB) );

    bgSum   = bgA.divide(bgB, 3, java.math.BigDecimal.ROUND_CEILING);
    dVal    = bgSum.doubleValue();

    return (dVal);
  }

  /** Method divide()
   *  This method will add two numbers and return the sum
   *
   *  dVal = dA / dB
   *
   *  @param  dA     = the variable A
   *  @param  dB     = the variable B
   *  @param  iRound = Rounding
   *  @return dVal   = the A divided by B
   */

  public static double divide( double dA, double dB, int iRound )
  {
    double     dVal  = 0.0;
    BigDecimal bgSum = new BigDecimal( "0.0" );
    BigDecimal bgA   = new BigDecimal( new String(""+dA) );
    BigDecimal bgB   = new BigDecimal( new String(""+dB) );

    bgSum   = bgA.divide(bgB, 3, iRound);
    dVal    = bgSum.doubleValue();

    return (dVal);
  }
}

/*
 *  @version 1.1 09/13/2013
 *  @author  John Victorine
 */
