/*
 * @productionPlotStruct.java Version 1.1 09/10/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package production.plot;

/** Class productionPlotStruct
 *  <p> This Class holds Production Plot Data
 *
 *  @version 1.1 09/10/2013
 *  @author  John R. Victorine
 */

public class productionPlotStruct
{
  public static final int    OIL       = 0;
  public static final int    GAS       = 1;

  public static final String PRODUCT[] = { "OIL", "GAS" };
  public static final String UNITS[]   = { "BBL", "MCF" };

  // Production Plot Parameters

  public static final int   _LINEAR    = 0; // Linear Axis Indicator
  public static final int   _LOG       = 1; // Log Axis Indicator

  public int    iWidth        = 0;       // Width of the plot area
  public int    iHeight       = 0;       // height of the plot area

  // --Titles

  public String sTitle1       = "";      // First line of the plot title
  public String sTitle2       = "";      // Second line of the plot title
  public String sTitle3       = "";      // Third line of the plot title

  // -- X-Axis ( Date Axis )

  public int    iXLog         = _LINEAR; // X-Axis Log Grid Indicator
  public int    iXCycles      = 1;       // Number of Log Cycles (Only of Log Scales)
  public double dXmax         = 0.0;     // Maximum X-Axis
  public double dXmin         = 0.0;     // Minimum X-Axis
  public double dXincr        = 0.0;     // Increment along X-Axis
  public String sXlabel       = "";      // X-Axis Label

  // -- Y-Axis ( Date Axis )

  public int    iYLog         = _LOG;    // Y-Axis Log Grid Indicator
  public int    iYCycles      = 1;       // Number of Log Cycles (Only of Log Scales)
  public double dYmax         = 0.0;     // Maximum Y-Axis
  public double dYmin         = 0.0;     // Minimum Y-Axis
  public double dYincr        = 0.0;     // Increment along Y-Axis
  public String sYlabel       = "";      // Y-Axis Label

  // Production Header Data

  public String sKID          = "0";     // DB Primary Key of the Header Record
  public String sName         = "";      // Well Name or Outcrop Name
  public String status        = iqstrat.iqstratStatusStruct.STATUS[0][0]; // Status

  // -- Location Information

  public String state         = "";      // State Name
  public int    iState        = -1;      // State Code
  public String sCounty       = "";      // County Name
  public int    iCounty       = -1;      // County Code

  public int    iTownship     = 0;       // Township
  public String sTownship     = "";      // Township Direction
  public int    iRange        = 0;       // Range
  public String sRange        = "";      // Range Direction
  public int    iSection      = 0;       // Section

  // -- XY Position Information

  public double dLatitude     = 0.0;     // Latitude of Location
  public double dLongitude    = 0.0;     // Longitude of Location
  public double dZone         = 0.0;     // UTM Zone
  public double dUTMx         = 0.0;     // UTM X-Position
  public double dUTMy         = 0.0;     // UTM Y-Position

  // Production Plot Data

  public int    iProduct      = -1;      // Product Identifier
  public int    iFactor       = 0;       // Normalization Factor for data

  public int    iRows         = 0;       // Number of Records
  public int    iTime[]       = null;    // Array of time values
  public double dProduction[] = null;    // Array of Production Values
  public double dCumulative[] = null;    // Array of Cumulative Production Values
  public int    iSources[]    = null;    // Array of Sources contributing

  public int    iRawRows      = 0;       // Number of Raw Records
  public double dTime[]       = null;    // Array of decimal time values
  public double dRaw[]        = null;    // Array of Raw Production Values

  /** Method delete()
   * <p> This method will set the arrays to null in order to get Java to
   *     deallocate memory for the object.
   */

  public void delete()
  {
    // Titles

    sTitle1 = "";  // First line of the plot title
    sTitle2 = "";  // Second line of the plot title
    sTitle3 = "";  // Third line of the plot title

    // X-Axis ( Date Axis )

    sXlabel  = "";      // X-Axis Label

    // Y-Axis ( Date Axis )

    sYlabel  = "";      // Y-Axis Label

    // Plot Data

    iRows       = 0;    // Number of Records
    iTime       = null; // Array of time values
    dProduction = null; // Array of Production Values
    dCumulative = null; // Array of Cumulative Production Values
    iSources    = null; // Array of Sources contributing

    iRawRows    = 0;    // Number of Raw Records
    dTime       = null; // Array of decimal time values
    dRaw        = null; // Array of Raw Production Values
  }
}

/*
 *  @version 1.1 09/10/2013
 *  @author  John Victorine
 */
