/*
 * @productionUtility.java Version 1.1 09/09/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package production;

import production.productionStruct;      // Production Data Structure
import production.productionListStruct;  // Production List Structure

/** Class productionUtility
 *  <p> This Class will provide basic utilities for the Rock Fossil
 *      data structures.
 *
 *  @version 1.1 09/09/2013
 *  @author  John R. Victorine
 */

public class productionUtility
{
  /** Method copyList()
   * <p> This method will copy one structure to another
   * @param  stOld = the Old List Structure
   * @return stNew = the New List structure
   */

  public static productionListStruct copyList(productionListStruct stOld)
  {
    productionListStruct stNew = null;

    if (stOld != null)
    {
      stNew        = new productionListStruct();

      stNew.sKID   = new String(stOld.sKID);  // Production Data Primary Key
      stNew.sID    = new String(stOld.sID);   // Other ID for the Production Data
      stNew.sType  = new String(stOld.sType); // Type of Production Data
      stNew.sName  = new String(stOld.sName); // Name of Production Data

      stNew.sApp   = new String(stOld.sApp);  // URL retrieving Data
      stNew.sBy    = new String(stOld.sBy);   // Search By

      stNew.iCount = stOld.iCount;
      stNew.stItem = new productionStruct[stOld.iCount];

      for (int i=0; i<stOld.iCount; i++)
      {
        stNew.stItem[i] = copy(stOld.stItem[i]);
      }
    }

    return (stNew);
  }

  /** Method transfer()
   * <p> This method will copy one structure to another
   * @param  stOld = the Old List Structure
   * @return stNew = the New List structure
   */

  public static productionListStruct transfer(productionListStruct stOld)
  {
    productionListStruct stNew = null;

    if (stOld != null)
    {
	  stNew = copyList( stOld );

	  stOld.delete();
	  stOld = null;
	}

    return (stNew);
  }

  /** Method copy()
   * <p> This method will copy data from one structure to another
   * @param  stOld = The original Fossil Data Structure
   * @return stNew = The Fossil Data Structure
   */

  public static productionStruct copy(productionStruct stOld)
  {
    productionStruct stNew = new productionStruct();

    if (stOld != null)
    {
      stNew.iYear    = stOld.iYear;      // Year of Production Data
      stNew.iMonth   = stOld.iMonth;     // Month of Production Data
      stNew.iQuarter = stOld.iQuarter;   // Quarter of Production Data
      stNew.date     = stOld.date;       // Date converted to a decimal number
      stNew.sProduct = new String(stOld.sProduct); // Product Name, Oil, Gas
      stNew.sUnits   = new String(stOld.sUnits);   // Units of Production Data
      stNew.data     = stOld.data;       // Production Value
      stNew.iSources = stOld.iSources;   // Number of Wells contributing to data
    }

    return (stNew);
  }

  /** Method add()
   * <p> This method will add the Production Data to an existing list
   * @param  stData = The Production Data Structure
   * @param  st     = The Old Production Data List Data Structure
   * @return st     = The new Production Data List Data Structure.
   */

  public static productionListStruct add( productionStruct stData,
                                          productionListStruct st )
  {
    int    i=0;
    int    iRecords = 0;
    int    iCount   = 0;
    int    iSource  = -1;
    productionListStruct stTemp = new productionListStruct();

    if (st != null)
      iRecords = st.iCount+1;
    else
      iRecords = 1;

    stTemp.sKID   = new String(st.sKID);  // Production Data Primary Key
    stTemp.sID    = new String(st.sID);   // Other ID for the Production Data
    stTemp.sType  = new String(st.sType); // Type of Production Data
    stTemp.sName  = new String(st.sName); // Name of Production Data

    stTemp.sApp   = new String(st.sApp);  // URL retrieving Data
    stTemp.sBy    = new String(st.sBy);   // Search By

    stTemp.stItem = new productionStruct[iRecords];

    if (st != null)
    {
      if (st.iCount > 0)
      {
        for (i=0; i<st.iCount; i++)
        {
          if (iCount < iRecords)
          {
            stTemp.stItem[iCount] = copy(st.stItem[i]);
            iCount++;
          }
        }
      }

      st.delete();
    }

    stTemp.stItem[iCount] = new productionStruct();
    stTemp.stItem[iCount] = copy(stData);
    iCount++;

    stTemp.iCount = iCount;

    st        = new productionListStruct();
    st.stItem = new productionStruct[iCount];
    st.iCount = iCount;

    st.sKID   = new String(stTemp.sKID);  // Production Data Primary Key
    st.sID    = new String(stTemp.sID);   // Other ID for the Production Data
    st.sType  = new String(stTemp.sType); // Type of Production Data
    st.sName  = new String(stTemp.sName); // Name of Production Data

    st.sApp   = new String(stTemp.sApp);  // URL retrieving Data
    st.sBy    = new String(stTemp.sBy);   // Search By

    for (i=0; i<stTemp.iCount; i++)
    {
      st.stItem[i] = copy(stTemp.stItem[i]);
    }

    stTemp.delete();

    return (st);
  }

  /** Method SumYear()
   * <p> This method will Sum Each Year of Production Data to an existing list
   * @param  st     = The Old Production Data List Data Structure
   * @return st     = The new Production Data List Data Structure.
   *

  public static productionListStruct SumYear( productionListStruct st )
  {
	int i, j;
	productionListStruct stPlot = null;
	int    iYear       = 0;
	int    iMinimum    = 0;
	int    iMaximum    = 0;
	int    iRows       = 0;
	int    iFound      = 0;
	String sProduct    = "";
	double dOil        = 0.0;
	double dGas        = 0.0;
	int    iSrcOil     = 0;
	int    iSrcGas     = 0;
	productionStruct stData = null;
System.out.println("A");

	if (st != null)
	{
System.out.println("B");
      for (i=0; i<st.iCount; i++)
      {
//		if (st.stItem[i].sProduct.equals(sProduct))
		{
		  if (iFound == 0)
		  {
			iMinimum = st.stItem[i].iYear;
			iMaximum = st.stItem[i].iYear;
			iFound = 1;
		  }

		  if (iMinimum > st.stItem[i].iYear)
		    iMinimum = st.stItem[i].iYear;

		  if (iMaximum < st.stItem[i].iYear)
		    iMaximum = st.stItem[i].iYear;
		}
	  }

	  iRows = iMaximum - iMinimum;
  	  iYear = iMinimum;
System.out.println(iMinimum+" "+iMaximum+" "+iRows);

	  for (i=0; i<iRows; i++)
	  {
	    dOil = 0.0;
	    dGas = 0.0;

		for (j=0; j<st.iCount; j++)
		{
	      if (iYear == st.stItem[j].iYear)
		  {
System.out.println(j+" "+iYear+" "+st.stItem[j].sProduct);
		    if (st.stItem[j].sProduct.equals("OIL"))
		    {
			  dOil    = dOil + st.stItem[j].data;
			  iSrcOil = st.stItem[j].iSources;
			}

		    if (st.stItem[j].sProduct.equals("GAS"))
		    {
			  dGas    = dGas + st.stItem[j].data;
			  iSrcGas = st.stItem[j].iSources;
			}
		  }
	    }
System.out.println(i+" "+iYear+" "+dOil+" "+dGas);

	    if (dOil > 0.0)
	    {
System.out.println("Ca");
	      stData = new productionStruct();
	      stData.iYear    = iYear;    // Year
	      stData.sProduct = "OIL";    // Product Name, Oil, Gas
	      stData.date     = (double) iYear;
	      stData.data     = dOil;     // Production Value
	      stData.iSources = iSrcOil;  // Number of Wells contributing to data
System.out.println("Cb");

	      stPlot = add( stData, stPlot );
System.out.println("C");

//	      if (stData != null)
//	        stData.delete();
//	      stData = null;
		}
System.out.println("D");

	    if (dGas > 0.0)
	    {
	      stData = new productionStruct();
	      stData.iYear    = iYear;    // Year
	      stData.sProduct = "GAS";    // Product Name, Oil, Gas
	      stData.date     = (double) iYear;
	      stData.data     = dGas;     // Production Value
	      stData.iSources = iSrcGas;  // Number of Wells contributing to data

	      stPlot = add( stData, stPlot );

//	      if (stData != null)
//	        stData.delete();
//	      stData = null;
		}
System.out.println("E");

	    iYear++;
System.out.println(iYear);
	  }
	}

    stPlot = bubbleSort( stPlot );

	return (stPlot);
  }
*/
  /** Method bubbleSort()
   * <p> This method will sort in ascending depth order (lowest to highest)
   * @param  st = data list structure
   * @return st = sorted data list structure
   */

  public static productionListStruct bubbleSort(productionListStruct st)
  {
    boolean swappedOnPrevRun = true;
    productionStruct stTemp   = null;

    if (st != null)
    {
      if (st.iCount > 1)
      {
        while(swappedOnPrevRun)
        {
          // this variable keeps track of whether to continue sorting or exit

          swappedOnPrevRun = false;

          // loop through every element in the array, except for the last one

          for(int i=0; i<st.iCount-1; i++)
          {
            // if current element is greater than the next swap the two elements

            if(st.stItem[i].date > st.stItem[i+1].date)
            {
              // we don't want the loop to end just yet, we're not done

              swappedOnPrevRun = true;

              // store element i in a temporary variable

              stTemp = copy(st.stItem[i]);

              // set element i+1 to where i used to be

              st.stItem[i] = copy(st.stItem[i+1]);

              // release the old i from temp into i+1 slot

              st.stItem[i+1] = copy(stTemp);
            }
          }
        }
      }
    }

    return (st);
  }

  public static void print(productionListStruct st)
  {
    if (st != null)
    {
      System.out.println("KEY:  " + st.sKID);   // KID for the Production Data
      System.out.println("DOR:  " + st.sID );   // Other ID for the Production Data
      System.out.println("Type: " + st.sType);  // Type of Production Data
      System.out.println("Name: " + st.sName);  // Name of Production Data

      System.out.println("URL:  " + st.sApp);   // URL retrieving Data
      System.out.println("By:   " + st.sBy );   // Search By

      for (int i=0; i<st.iCount; i++)
      {
        System.out.println( st.stItem[i].iYear    + " " +
                            st.stItem[i].iMonth   + " " +
                            st.stItem[i].iQuarter + " " +
                            st.stItem[i].date     + " " +
                            st.stItem[i].sProduct + " " +
                            st.stItem[i].data     + " " +
                            st.stItem[i].iSources );
      }
    }
  }
}

/*
 *  @version 1.1 09/09/2013
 *  @author  John Victorine
 */
