/*
 * @utilPlotLandGrid.java Version 1.1 12/31/2002
 *
 * Copyright (c) 2001 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package util;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.Graphics2D.*;
import javax.swing.*;

import util.utilLandGridListStruct;

/** Class utilPlotLandGrid
 *  <p> This class will draw the Section Township Range Land Grid
 *
 *  @version 1.1 12/31/2002
 *  @author  John R. Victorine
 */

public class utilPlotLandGrid extends Canvas
{
  public static final int        COUNTY_GRID = 0;
  public static final int        AREA_GRID   = 1;
  public static final int        NO_TEXT     = 2;

  /* Grid Map
   * -- County map
   *      Latitude  iHeight / 1 deg latitude
   *      Longitude (4 * iHeight / 5)  / 1 deg longitude
   * -- Area  map
   *      Latitude  iHeight / 2.5 deg latitude
   *      Longitude (4 * iHeight / 5)  / 2.5 deg longitude
   */

  private int                    iGrid       = NO_TEXT;

  // Plot Dimensions

  private utilLandGridListStruct stLandGrid  = null;
  private int                    iXBegin     = 0;
  private int                    iYBegin     = 0;
  private int                    iWidth      = 0;
  private int                    iHeight     = 0;
  private double                 dXMax       = 0.0;
  private double                 dXMin       = 0.0;
  private double                 dYMax       = 0.0;
  private double                 dYMin       = 0.0;

  /** Constructor utilPlotLandGrid()
   *  <p> This is the Constructor for this class.
   *  @param st     - Land Grid List Data Structure.
   *  @param width  - Width of the Well Map
   *  @param height - Height of the Well Map
   *  @param dXL    - Lower X Limit of the Land Grid
   *  @param dXU    - Upper X Limit of the Land Grid
   *  @param dYL    - Lower Y Limit of the Land Grid
   *  @param dYU    - Upper Y Limit of the Land Grid
   *  @param iGrid  - Plot Text with Grid or NOT.
   */

  public utilPlotLandGrid( utilLandGridListStruct st, int width, int height,
                           double dXL, double dXU, double dYL, double dYU,
                           int iGrid )
  {
    try
    {
      stLandGrid    = st;

      iXBegin       = 0;
      iYBegin       = 0;
      iWidth        = width-iXBegin;
      iHeight       = height-iYBegin;

      dXMin = dXL;
      dXMax = dXU;
      dYMin = dYL;
      dYMax = dYU;

      this.iGrid = iGrid;
    }
    catch (Exception e)
    {
      JOptionPane.showMessageDialog((Component) null,
                                    "utilPlotLandGrid(): " + e.getMessage(),
                                    "ERROR",
                                    JOptionPane.ERROR_MESSAGE);
    }
  }

  /** Constructor utilPlotLandGrid()
   *  <p> This is the Constructor for this class.
   *  @param st     - Land Grid List Data Structure.
   *  @param width  - Width of the Well Map
   *  @param height - Height of the Well Map
   *  @param dXL    - Lower X Limit of the Land Grid
   *  @param dXU    - Upper X Limit of the Land Grid
   *  @param dYL    - Lower Y Limit of the Land Grid
   *  @param dYU    - Upper Y Limit of the Land Grid
   *  @param iGrid  - Plot Text with Grid or NOT.
   */

  public utilPlotLandGrid( utilLandGridListStruct st, int width, int height, int iGrid )
  {
    try
    {
      stLandGrid    = st;

      iXBegin       = 0;
      iYBegin       = 0;
      iWidth        = width-iXBegin;
      iHeight       = height-iYBegin;

      getMinMaxPosition();

      this.iGrid = iGrid;
    }
    catch (Exception e)
    {
      JOptionPane.showMessageDialog((Component) null,
                                    "utilPlotLandGrid(): " + e.getMessage(),
                                    "ERROR",
                                    JOptionPane.ERROR_MESSAGE);
    }
  }

  /* ======================================================================== *
   * ------------------------- GET Methods ---------------------------------- *
   * ======================================================================== */

  /** Method getMinMaxPosition()
   *  <p> This method will determine the minimum and maximum position of the
   *      wells that have been passed in.
   */

  private void getMinMaxPosition()
  {
    int    i          = 0;
    int    j          = 0;
    int    iTemp      = 0;
    int    iFound     = 0;
    int    iStart     = 0;
    double dLongitude = 0.0;
    double dLatitude  = 0.0;
    double dMinLat    = 0.0;
    double dMaxLat    = 0.0;
    double dMinLong   = 0.0;
    double dMaxLong   = 0.0;

    for (i=0; i<stLandGrid.iCount; i++)
    {
      for (j=0; j<stLandGrid.stItem[i].iTotal; j++)
      {
        if (iStart == 0)
        {
          dMinLat  = stLandGrid.stItem[i].dLatitude[j];
          dMaxLat  = stLandGrid.stItem[i].dLatitude[j];
          dMinLong = stLandGrid.stItem[i].dLongitude[j];
          dMaxLong = stLandGrid.stItem[i].dLongitude[j];
          iStart   = 1;
        }

        if (dMinLat > stLandGrid.stItem[i].dLatitude[j])
          dMinLat  = stLandGrid.stItem[i].dLatitude[j];

        if (dMaxLat < stLandGrid.stItem[i].dLatitude[j])
          dMaxLat  = stLandGrid.stItem[i].dLatitude[j];

        if (dMinLong > stLandGrid.stItem[i].dLongitude[j])
          dMinLong  = stLandGrid.stItem[i].dLongitude[j];

        if (dMaxLong < stLandGrid.stItem[i].dLongitude[j])
          dMaxLong  = stLandGrid.stItem[i].dLongitude[j];
      }
    }
/*
    for (i=0; i<iRows; i++)
    {
      dLatitude  = stPlot.stItem[i].dLatitude;
      dLongitude = stPlot.stItem[i].dLongitude;

      if ((Math.abs(dLongitude) > 0.0) &&
          (Math.abs(dLatitude)  > 0.0))
      {
        if (iFound == 0)
        {
          dXMin = dLongitude;
          dXMax = dXMin;

          dYMin = dLatitude;
          dYMax = dYMin;

          iFound = 1;
        }

        if (dLongitude < dXMin)
          dXMin = dLongitude;

        if (dLongitude > dXMax)
          dXMax = dLongitude;

        if (dLatitude < dYMin)
          dYMin = dLatitude;

        if (dLatitude > dYMax)
          dYMax =dLatitude;
      }
    }

    if (dXMin < dMinLong)
      dXMin = dMinLong;

    if (dXMax > dMaxLong)
      dXMax = dMaxLong;

    if (dYMin < dMinLat)
      dYMin = dMinLat;

    if (dXMax > dMinLat)
      dYMax = dMaxLat;

    if (IN == 0)
    {
*/
      dXMin = dMinLong;
      dXMax = dMaxLong;
      dYMin = dMinLat;
      dYMax = dMaxLat;
//    }

    double dX = 0.05 * (dXMax - dXMin);
    double dY = 0.05 * (dYMax - dYMin);

    dXMin = dXMin - dX;
    dXMax = dXMax + dX;
    dYMin = dYMin - dY;
    dYMax = dYMax + dY;
  }

  /* ======================================================================== *
   * ------------------------- SET Methods ---------------------------------- *
   * ======================================================================== */

  /** Method setXYLimit()
   *  <p> This method will reset the limits of the XY Position of the land grid.
   *
   *  @param dXL    - Lower X Limit of the Land Grid
   *  @param dXU    - Upper X Limit of the Land Grid
   *  @param dYL    - Lower Y Limit of the Land Grid
   *  @param dYU    - Upper Y Limit of the Land Grid
   */

  public void setXYLimt(double dXL, double dXU, double dYL, double dYU)
  {
    dXMin = dXL;
    dXMax = dXU;
    dYMin = dYL;
    dYMax = dYU;
  }

  /* ======================================================================== *
   * ------------------------- DRAW Methods --------------------------------- *
   * ======================================================================== */

  /** Method drawGrid()
   *  <p> This method will create a grid of the Latitude and Longitude lines.
   *  @param g - Graphics pointer.
   */

  public void drawGrid(Graphics g)
  {
    int    iXo = 0;
    int    iYo = 0;
    int    iTemp  = 0;
    int    iRatio = 0;
    double dX = 0.0;
    double dY = 0.0;

    dX = dXMin * 100.0;
    iTemp = (int) dX;
    dX = ((double) iTemp) / 100.0;

    dY = dXMin * 100.0;
    iTemp = (int) dY;
    dY = ((double) iTemp) / 100.0;

    while (dX < dXMax)
    {
      iRatio = (int)((double)iWidth * (dX-dXMin) / (dXMax - dXMin));
      iXo = iXBegin + iRatio;

      if ((iXo >= iXBegin) && (iXo <= iXBegin+iWidth))
      {
        g.setColor(Color.lightGray);
        g.drawLine(iXo, iYBegin, iXo, iYBegin + iHeight);
      }

      dX = dX + 0.01;
    }

    while (dY < dYMax)
    {
      iRatio = (int)((double)iHeight * (dY-dYMin) / (dYMax - dYMin));
      iYo = iYBegin + iHeight - iRatio;

      if ((iYo >= iYBegin) && (iYo <= iYBegin+iHeight))
      {
        g.setColor(Color.lightGray);
        g.drawLine(iXBegin, iYo, iXBegin+iWidth, iYo);
      }

      dY = dY + 0.01;
    }
  }

  /** Method drawSTRGrid()
   *  <p> This method will draw the Section, Township and Range Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawSTRGrid(Graphics g)
  {
    int iRatio = 0;
    int iXo[] = null;
    int iYo[] = null;
    int iXMid = 0;
    int iYMid = 0;
    String str = "";

    if (stLandGrid.iCount > 0)
    {
      for (int i=0; i<stLandGrid.iCount; i++)
      {
        if (stLandGrid.stItem[i].iTotal > 0)
        {
          iXo = new int[stLandGrid.stItem[i].iTotal];
          iYo = new int[stLandGrid.stItem[i].iTotal];

          iXMid = 0;
          iYMid = 0;

          for (int j=0; j<stLandGrid.stItem[i].iTotal; j++)
          {
            iRatio = (int)((double)iWidth *
                         (stLandGrid.stItem[i].dLongitude[j]-dXMin) /
                         (dXMax - dXMin));
            iXo[j] = iXBegin + iRatio;

            if (j==0)
              iXMid = iXo[j];
            else if (iXMid > iXo[j])
              iXMid = iXo[j];

            iRatio = (int)((double)iHeight *
                         (stLandGrid.stItem[i].dLatitude[j] - dYMin) /
                         (dYMax - dYMin));
            iYo[j] = iYBegin + iHeight - iRatio;

            if (j==0)
              iYMid = iYo[j];
            else if (iYMid > iYo[j])
              iYMid = iYo[j];
          }

          if (iGrid == AREA_GRID)
            g.setColor(new Color(240, 240, 255));
          else
            g.setColor(Color.lightGray);

//          g.setColor(Color.lightGray);
          g.setFont( new Font("Monospaced", Font.BOLD, 9) );
          if (iGrid != AREA_GRID)
          {
//            g.drawString(stLandGrid.stItem[i].sTownship, iXMid, iYMid + 10);
//            g.drawString(stLandGrid.stItem[i].sRange,    iXMid, iYMid + 20);
//            g.drawString(stLandGrid.stItem[i].section,   iXMid, iYMid + 30);
            str = new String(stLandGrid.stItem[i].section.substring(1,stLandGrid.stItem[i].section.length()));
            g.drawString(str,   iXMid+1, iYMid + 10);
//            g.drawString(stLandGrid.stItem[i].section,   iXMid, iYMid + 10);
          }
          g.drawPolygon(iXo, iYo, stLandGrid.stItem[i].iTotal);
        }
      }
    }
  }

  /** Method drawMajorSTRGrid()
   *  <p> This method will draw the Section, Township and Range Grid Lines
   *  @param g - Graphics pointer.
   */

  public void drawMajorSTRGrid(Graphics g)
  {
    int i      = 0;
    int j      = 0;
    int iRatio = 0;
    int iXo[]  = null;
    int iYo[]  = null;
    int iXMid  = 0;
    int iYMid  = 0;

    if (stLandGrid.iCount > 0)
    {
      for (i=0; i<stLandGrid.iCount; i++)
      {
        if (stLandGrid.stItem[i].iTotal > 0)
        {
          iXo = new int[stLandGrid.stItem[i].iTotal];
          iYo = new int[stLandGrid.stItem[i].iTotal];

          iXMid = 0;
          iYMid = 0;

          for (j=0; j<stLandGrid.stItem[i].iTotal; j++)
          {
            iRatio = (int)((double)iWidth *
                         (stLandGrid.stItem[i].dLongitude[j]-dXMin) /
                         (dXMax - dXMin));
            iXo[j] = iXBegin + iRatio;

            if (j==0)
              iXMid = iXo[j];
            else if (iXMid > iXo[j])
              iXMid = iXo[j];

            iRatio = (int)((double)iHeight *
                         (stLandGrid.stItem[i].dLatitude[j] - dYMin) /
                         (dYMax - dYMin));
            iYo[j] = iYBegin + iHeight - iRatio;

            if (j==0)
              iYMid = iYo[j];
            else if (iYMid > iYo[j])
              iYMid = iYo[j];
          }

          if (iGrid == AREA_GRID)
            g.setColor(Color.gray);
          else
            g.setColor(Color.darkGray);

          g.setFont( new Font("Monospaced", Font.BOLD, 11) ); //9) );

          if ((iXMid >= iXBegin) && (iXMid <= iXBegin+iWidth) &&
              (iYMid >= iYBegin) && (iYMid <= iYBegin+iHeight))
          {
            if (iGrid != NO_TEXT)
            {
              if (stLandGrid.stItem[i].section.equals("S 0"))
              {
                g.drawString(stLandGrid.stItem[i].sTownship,
                             iXMid + 2,
                             iYMid + 10);
                g.drawString(stLandGrid.stItem[i].sRange,
                             iXMid + 2,
                             iYMid + 20);
              }
            }

            if (stLandGrid.stItem[i].section.equals("S 0"))
              g.drawPolygon(iXo, iYo, stLandGrid.stItem[i].iTotal);
          }
        }
      }
    }
  }

  /** Method paint()
   *  <p> This method will create the Linear Plot.
   *  @param g - Graphics pointer.
   */

  public void paint(Graphics g)
  {
    g.setColor(Color.white);
    g.fillRect(0, 0, iWidth, iHeight);

    if (stLandGrid.iCount > 0)
    {
      drawSTRGrid(g);
      drawMajorSTRGrid(g);
    }
    else
    {
      drawGrid(g);
      drawMajorSTRGrid(g);
    }
  }
}
