/*
 * @utilPlotLandGridImage.java Version 1.1 12/31/2002
 *
 * Copyright (c) 2001 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package util;

import java.awt.*;
import java.awt.geom.*;
import java.io.*;
import javax.swing.*;

import util.utilLandGridListStruct;
import util.utilPlotLandGrid;

/** Class utilPlotLandGridImage
 *  <p> This Class will create the Land Grid Plot Image.
 *
 *  @version 1.1 12/31/2002
 *  @author  John R. Victorine
 */

public class utilPlotLandGridImage
{
  /** Method makePlotImage()
   *  This method will plot Well Symbol List data. The plot is then converted to
   *  an image and passed back to the calling routine.
   *  @param st      = Land Grid List Data Structure.
   *  @param iWidth  = The width of image
   *  @param iHeight = The height of the image
   *  @param dXL     = Lower X Limit of the Land Grid
   *  @param dXU     = Upper X Limit of the Land Grid
   *  @param dYL     = Lower Y Limit of the Land Grid
   *  @param dYU     = Upper Y Limit of the Land Grid
   *  @param iGrid   = Plot Text with Grid or NOT.
   */

  public static Image makePlotImage(utilLandGridListStruct st,
                                    int iWidth,
                                    int iHeight,
                                    double dXL, double dXU,
                                    double dYL, double dYU,
                                    int iGrid)
  {
    Frame frame = new Frame();
    frame.addNotify();

    utilPlotLandGrid plotLandGrid =
        new utilPlotLandGrid(st, iWidth, iHeight, dXL, dXU, dYL, dYU, iGrid);

    Image messageImage = frame.createImage(iWidth, iHeight);
    Graphics graphics = messageImage.getGraphics();

    graphics.setColor(Color.white);
    graphics.fillRect(0, 0, iWidth, iHeight);

    if (st.iCount > 0)
    {
      plotLandGrid.drawSTRGrid(graphics);
      plotLandGrid.drawMajorSTRGrid(graphics);
    }
    else
    {
      plotLandGrid.drawGrid(graphics);
      plotLandGrid.drawMajorSTRGrid(graphics);
    }

    graphics.dispose();

    return (messageImage);
  }
}
